/*****************************************************************************
 *
 * FILE:	alias.c
 * DESCRIPTION:	register Redirect/Alias/ScriptAlias
 * DATE:	Sun, Sep 26 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2001 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: alias.c,v 1.46.2.4 2002/01/30 13:26:33 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <stdlib.h>
#include <string.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static String		checkAlias(String, alias_t);
static unsigned int	hash(String);


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	registerAliases
 * Description:		Registers a Redirect, Alias, or ScriptAlias to the
 *			alias hash table.
 * Arguments:		fakeName, realName, alias_type
 * Return Value:	none
 */
void
registerAliases(fakeName, realName, type)
	String	fakeName;
	String	realName;
	alias_t	type;
{
  int		h;	/* hash value	*/
  Alias *	ap;	/* current alias pointer	*/

  if (checkAlias(fakeName, type) != NULL) {
    return;	/* already registered */
  }

  ap = (Alias *)xcalloc(1, sizeof(Alias));
  if (ap == NULL) {
    die(ERR_OUT_OF_MEMORY);
  }

  h				= hash(fakeName);
  ap->type			= type;
  ap->fakeName			= xstrdup(fakeName);
  ap->fakeLen			= strlen(fakeName);
  ap->realName			= xstrdup(realName);
  ap->realLen			= strlen(realName);
  ap->next			= httpd->vhost->aliasTable[h];
  httpd->vhost->aliasTable[h]	= ap;
}

/*
 * Function Name:	checkAlias
 * Description:		check the Redirect, Alias, or ScriptAlias from the
 *			alias hash table.
 * Arguments:		fakeName, alias_type
 * Return Value:	realName
 */
static String
checkAlias(fakeName, type)
	String	fakeName;
	alias_t	type;
{
  register Alias *	ap;

  for (ap = httpd->vhost->aliasTable[hash(fakeName)]; ap != NULL;
       ap = ap->next) {
    if (strcmp(ap->fakeName, fakeName) == 0) {
      return ap->realName;
    }
  }
  return NULL;
}


/*
 * Function Name:	parseURI
 * Description:		parse a requests URI.
 * Arguments:		thread
 * Return Value:	Error/CGI is FALSE, otherwise TRUE
 */
bool_t
parseURI(thd)
	register Thread *	thd;
{
  register String	urip;
  register Alias *	ap;
  String		p;

  /*
   * remove CRLF, CR or LF code from line end, and
   * set a query string.
   */
  for (urip = chomp(thd->requestURI); *urip != EOL && *urip != '?'; urip++) {
    ;
  }
  if (*urip == '?') {
    *urip++ = EOL;	/* overwrite '?' to EOL */
    thd->cgi.query_string = xstrdup(urip);
  }

  /*
   * parse a percent-encoded URI (URI length is short after decoding)
   */
  (void)decodeEscapedURI(thd->requestURI);

  /*
   * Forbid a request URI with relative path
   */
  urip = (String)strstr(thd->requestURI, "/..");
  if (urip != NULL && (urip[3] == '/' || urip[3] == EOL)) {
    setResponseCode(thd, CODE_FORBIDDEN);
    return FALSE;
  }

  /*
   * find ScriptAlias, Alias, or Redirect
   */
  for (ap = httpd->vhost->aliasTable[hash(thd->requestURI)];
	ap != NULL; ap = ap->next) {
    int	n;
    /* aliased URI */
    if (strncmp(thd->requestURI, ap->fakeName, ap->fakeLen) == 0 ||
	strncmp(thd->requestURI, ap->fakeName, ap->fakeLen - 1) == 0) {
      switch (ap->type) {
	/* Redirection */
	case A_REDIRECT:
	  thd->response.location = xstrdup(ap->realName);
	  setResponseCode(thd, CODE_MOVED_PERMANENTLY);
	  return FALSE;
	/* Alias */
	case A_ALIAS:
	  if (xasprintf(&thd->path, "%s%s",
			ap->realName, thd->requestURI + ap->fakeLen) == -1) {
	    setResponseCode(thd, CODE_INTERNAL_SERVER_ERROR);
	    wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
	    return FALSE;
	  }
	  setResponseCode(thd, CODE_REQUEST_OK);
	  break;
	/* Script Alias */
	case A_SCRIPT:
	  thd->state = CGI_READY;
	  thd->ctype = C_CGI;
	  urip = (String)strchr(thd->requestURI + ap->fakeLen, '/');
	  if (urip != NULL) {
	    thd->cgi.path_info = xstrdup(urip);
	    *urip = EOL;
	  }
	  n = xasprintf(&thd->path, "%s%s",
			ap->realName, thd->requestURI + ap->fakeLen);
	  if (n == -1) {
	    setResponseCode(thd, CODE_INTERNAL_SERVER_ERROR);
	    wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
	    return FALSE;
	  }
	  /* directory listing for cgi-bin directory */
	  if (thd->path[n - 1] == '/') {
	    setResponseCode(thd, CODE_BAD_REQUEST);
	    return FALSE;
	  }

	default:
	  break;
      }
    }
  }

  /*
   * find UserDir
   */
  if (thd->ctype == C_UNK && httpd->vhost->userDir != NULL &&
      thd->requestURI[1] == '~') {
    size_t    len =  strcspn(thd->requestURI + TILDE_OFFSET, "/");
    thd->userName = xstrndup(thd->requestURI + TILDE_OFFSET, len);
    thd->userHome = getHomeDir(thd->userName);
    if (thd->userHome == NULL) {
      setResponseCode(thd, CODE_NOT_FOUND);
      return FALSE;
    }

    /* actual path of request file */
    if (xasprintf(&thd->path, "%s/%s%s",
		  thd->userHome, httpd->vhost->userDir,
		  thd->requestURI + TILDE_OFFSET + len) == -1 ||
	/* actual path of user directory (i.e. public_html) */
	xasprintf(&thd->userDir, "%s/%s",
		  thd->userHome, httpd->vhost->userDir) == -1) {
      setResponseCode(thd, CODE_INTERNAL_SERVER_ERROR);
      wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
      return FALSE;
    }
    setResponseCode(thd, CODE_REQUEST_OK);
  }

  /*
   * request URI is relative to DOCROOT
   */
  if (thd->ctype == C_UNK) {
    if (xasprintf(&thd->path, "%s%s",
		  httpd->vhost->docRoot, thd->requestURI) == -1) {
      setResponseCode(thd, CODE_INTERNAL_SERVER_ERROR);
      wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
      return FALSE;
    }
    setResponseCode(thd, CODE_REQUEST_OK);
  }

  /*
   * authentication files
   */
  p = strrchr(thd->requestURI, '/');
  if (p != NULL) {
    p++;
    if (strcmp(p, USER_ACCESS) == 0 || strcmp(p, USER_PASSWD) == 0) {
      setResponseCode(thd, CODE_FORBIDDEN);
      return FALSE;
    }
  }

  return TRUE;
}


/*
 * Function Name:	hash
 * Description:		calculates the hash value of string
 * Arguments:		string
 * Return Value:	hash value
 */
static unsigned int
hash(str)
	String	str;
{
  register unsigned char *	p;
  unsigned int			h;
  p = (unsigned char *)str;
  for (h = *p++; *p != '/' && *p != EOL; p++) {
    h += *p;
  }

  return h % ALIAS_TABLE_SIZE;
}

