%{
/*****************************************************************************
 *
 * FILE:	conf_lexer.l
 * DESCRIPTION:	configuration lexical analyzer
 * DATE:	Thu, Sep  9 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: conf_lexer.l,v 1.13 2002/01/13 04:05:20 kouichi Exp $
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "conf_grammar.h"

#ifdef	yywrap	/* defined as macro in old flex */
#undef	yywrap
#endif	/* yywrap */

static int	mime  = 0;	/* for mime.types	*/
int		lines = 1;	/* line number counter	*/

extern int	yywrap(void);
extern void	yyerror(char *);
%}

WS	[ \t]+
MTYPE	[^ \t\n]+\/[^ \t\n]+

%x VHOST
%x MIME
%%

<MIME>\"?{MTYPE}\"?|\"{MTYPE};{WS}[^ \t]+\"	{
		  if (mime == 0) BEGIN(INITIAL);
		  yylval.sval = yytext; return MIMETYPE;
		}

<*>{WS}		;	/* ignore whitespace	*/
<*>#.*		;	/* ignore comment	*/

<INITIAL>Port			{ BEGIN(INITIAL); return W_PORT; }
<INITIAL>SecurePort		{ BEGIN(INITIAL); return W_SECUREPORT; }
<INITIAL>MaxRequest		{ BEGIN(INITIAL); return W_MAXREQUEST; }
<INITIAL>Timeout		{ BEGIN(INITIAL); return W_TIMEOUT; }
<INITIAL>User			{ BEGIN(INITIAL); return W_USER; }
<INITIAL>Group			{ BEGIN(INITIAL); return W_GROUP; }
<INITIAL>PidFile		{ BEGIN(INITIAL); return W_PIDFILE; }
<INITIAL>ServerRoot		{ BEGIN(INITIAL); return W_SERVERROOT; }

<INITIAL,VHOST>ErrorLog		{ BEGIN(YY_START); return W_ERRORLOG; }
<INITIAL,VHOST>AccessLog	{ BEGIN(YY_START); return W_ACCESSLOG; }
<INITIAL,VHOST>ServerAdmin	{ BEGIN(YY_START); return W_SERVERADMIN; }
<INITIAL>ServerName		{ BEGIN(INITIAL); return W_SERVERNAME; }

<INITIAL,VHOST>DocumentRoot	{ BEGIN(YY_START); return W_DOCUMENTROOT; }
<INITIAL,VHOST>UserDir		{ BEGIN(YY_START); return W_USERDIR; }
<INITIAL,VHOST>DirectoryIndex	{ BEGIN(YY_START); return W_DIRECTORYINDEX; }
<INITIAL,VHOST>FollowSymLinks	{ BEGIN(YY_START); return W_FOLLOWSYMLINKS; }
<INITIAL,VHOST>DefaultType	{ BEGIN(YY_START); return W_DEFAULTTYPE; }

<INITIAL>ErrorDocument		{ BEGIN(INITIAL); return W_ERROR_DOCUMENT; }

<INITIAL>AddType		{ BEGIN(MIME); return W_ADDTYPE; }

<INITIAL,VHOST>Redirect		{ BEGIN(YY_START); return W_REDIRECT; }
<INITIAL,VHOST>Alias		{ BEGIN(YY_START); return W_ALIAS; }
<INITIAL,VHOST>ScriptAlias	{ BEGIN(YY_START); return W_SCRIPTALIAS; }

<INITIAL>AddModule		{ BEGIN(INITIAL); return W_MODULE; }

<INITIAL>BindAddress		{ BEGIN(INITIAL); return W_BINDADDRESS; }

<INITIAL>"<VirtualHost"({WS}.*)">"	{
		    register char * sp = strchr(yytext, '"');
		    register char * ep;
		    if (sp != NULL) {
		      ep = strchr(++sp, '"');
		      if (ep != NULL) {
			*ep = '\0';
			yylval.sval = sp;
			BEGIN(VHOST);
			return W_BEGIN_VIRTUALHOST;
		      }
		    }
		    yyerror("no virtual server name");
		    exit(1);
		}

<VHOST>"</VirtualHost>"	{ BEGIN(INITIAL); return W_END_VIRTUALHOST; }

<*>[0-9]+	{ yylval.ival = atoi(yytext); return INTEGER; }
<*>[^ \t\n]+	{ yylval.sval = yytext; return STRING; }

<*>\n		{ lines++; }
%%

/*
 *	overwrite yywrap()
 */
int
yywrap(void)
{
  fclose(yyin);
  mime++;

  switch (mime) {
    case 1:
      yyin = fopen("conf/mime.types", "r");
      if (yyin == NULL) {
	fputs("Cannot open mime.types file.\n"
	      "Please confirm ServerRoot in wyvern.conf.\n", stderr);
	exit(1);
      }
      BEGIN MIME;
      lines = 1;
      return 0;	/* continue reading */
    default:
      BEGIN INITIAL;
      mime = 0;	/* in case we reread config files	*/
      return 1;	/* EOF */
  }
}

/*
 *	overwrite yyerror()
 */
void
yyerror(msg)
	char *	msg;
{
  fprintf(stderr, "Error on line %d of %s: %s\n", lines,
		(mime == 0 ? "wyvern.conf" : "mime.types"), msg);
}
