/*****************************************************************************
 *
 * FILE:	log.c
 * DESCRIPTION:	record server logging
 * DATE:	Sun, Sep 26 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: log.c,v 1.33.2.2 2002/01/27 16:09:19 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	__STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif	/* __STDC__ */
#include <string.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <time.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros
 *
 *****************************************************************************/
#ifndef	HAVE_SETLINEBUF
#define	setlinebuf(x)	setvbuf((x), (char *)NULL, _IOLBF, 0)
#endif	/* HAVE_SETLINEBUF */


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
#ifndef	USE_SYSLOG
static void	subsyslog(log_t, const char *, va_list);
#endif	/* USE_SYSLOG */


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	openLogFiles
 * Description:		open log files
 * Arguments:		none
 * Return Value:	none
 */
void
openLogFiles(void)
{
  register VirtualHost *	vhost;

  for (vhost = httpd->host; vhost != NULL; vhost = vhost->next) {
#ifndef	USE_SYSLOG
    /* open error-log file */
    if (vhost->errorLog.path == NULL) {
      vhost->errorLog.fp = httpd->host->errorLog.fp;
    }
    else {
      vhost->errorLog.fp = fopen(vhost->errorLog.path, "a");
      if (vhost->errorLog.fp == NULL) {
	die("failed to open errorLog: %s", vhost->errorLog.path);
      }
      (void)setlinebuf(vhost->errorLog.fp);	/* ignore error */
      (void)fchown(fileno(vhost->errorLog.fp), httpd->uid, httpd->gid);
    }
#endif	/* USE_SYSLOG */

    /* open access-log file */
    if (vhost->accessLog.path == NULL) {
      vhost->accessLog.fp = httpd->host->accessLog.fp;
    }
    else {
      vhost->accessLog.fp = fopen(vhost->accessLog.path, "a");
      if (vhost->accessLog.fp == NULL) {
	die("failed to open accessLog: %s", vhost->accessLog.path);
      }
      (void)setlinebuf(vhost->accessLog.fp);	/* ignore error */
      (void)fchown(fileno(vhost->accessLog.fp),  httpd->uid, httpd->gid);
    }
  }

  /* switch default log files */
#if	USE_SYSLOG
  httpd->accessLog = httpd->host->accessLog.fp;
#else
  httpd->errorLog  = httpd->host->errorLog.fp;
  httpd->accessLog = httpd->host->accessLog.fp;
#endif	/* USE_SYSLOG */
}

/*
 * Function Name:	closeLogFiles
 * Description:		close log files
 * Arguments:		none
 * Return Value:	none
 */
void
closeLogFiles(void)
{
  register VirtualHost *	vhost;

  for (vhost = httpd->host; vhost != NULL; vhost = vhost->next) {
#ifndef	USE_SYSLOG
    /* close error-log file */
    if (vhost->errorLog.path != NULL && vhost->errorLog.fp != NULL) {
      fclose(vhost->errorLog.fp);
      vhost->errorLog.fp = NULL;
    }
#endif	/* USE_SYSLOG */

    /* close access-log file */
    if (vhost->accessLog.path != NULL && vhost->accessLog.fp != NULL) {
      fclose(vhost->accessLog.fp);
      vhost->accessLog.fp = NULL;
    }
  }
}

/*
 * Function Name:	writeLogs
 * Description:		write the log message in log file
 * Arguments:		thread
 * Return Value:	none
 */
void
writeLogs(thd)
	register Thread *	thd;
{
  /* Informational: 1xx */
  /* Successful:    2xx */
  /* Redirection:   3xx */
  if (thd->rsCode < CODE_BAD_REQUEST) {
    if (thd->request.x_forwarded_for != NULL) {
      fprintf(httpd->accessLog, "%s(%s) ",
	      thd->client.name == NULL ? thd->client.ip : thd->client.name,
	      thd->request.x_forwarded_for);

    }
    else {
      fprintf(httpd->accessLog, "%s ",
	      thd->client.name == NULL ? thd->client.ip : thd->client.name);
    }
    fprintf(httpd->accessLog, "%s %s [%s] \"%s %s %s\" %d ",
	    thd->identUser == NULL ? "-" : thd->identUser,
	    thd->authUser == NULL ? "-" : thd->authUser,
	    getTimeOfRFC822(thd->atime),
	    thd->method.name,
	    thd->requestURI,
	    thd->http.protocol,
	    getResponseCode(thd->rsCode));
    if (thd->rsCode > CODE_PARTIAL_CONTENT) {	/* for Redirection: 3xx */
	fputs("-", httpd->accessLog);
    }
    else {
      if (thd->method.type == M_HEAD || thd->method.type == M_GET) {
#if	HAVE_ZLIB_H
	fprintf(httpd->accessLog, "%ld (%4.1f%%)",
		(long)thd->file.st_size, thd->gzippct);
#else
	fprintf(httpd->accessLog, "%ld", (long)thd->file.st_size);
#endif	/* HAVE_ZLIB_H */
      }
      else if (thd->method.type == M_POST) {
	fprintf(httpd->accessLog, "%d", thd->request.content_length);
      }
      else {
	fputs("-", httpd->accessLog);
      }
    }
    fputs("\n", httpd->accessLog);
  }
  /* Client Error: 4xx */
  /* Server Error: 5xx */
  else {
    if (thd->request.x_forwarded_for != NULL &&
	strncasecmp(thd->request.x_forwarded_for, "unknown", 7) == 0) {
      wvSyslog(LOG_WARNING,
		"(%s) \"%s %s %s\" request forwarded from unknown host.\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		thd->method.name,
		thd->requestURI,
		thd->http.protocol);
    }
    else if (thd->method.name == NULL || thd->requestURI[0] == EOL) {
      wvSyslog(LOG_EMERG, "attacked or scanned from %s; %s (%d)\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		getResponsePhrase(thd->rsCode),
		getResponseCode(thd->rsCode));
    }
    else {
      wvSyslog(LOG_ERR, "%s \"%s %s %s\" %s (%d)\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		thd->method.name,
		thd->requestURI,
		thd->http.protocol,
		getResponsePhrase(thd->rsCode),
		getResponseCode(thd->rsCode));
    }
  }
}

/*
 * Function Name:	die
 * Description:		print an error message and terminate process
 * Arguments:		type of error
 * Return Value:	none
 */
void
#if	__STDC__
die(const char *fmt, ...)
#else
die(fmt, va_alist)
	char *	fmt;
	va_dcl
#endif	/* __STDC__ */
{
  va_list	ap;
#if	__STDC__
  va_start(ap, fmt);
#else
  va_start(ap);
#endif	/* __STDC__ */
#if	USE_SYSLOG
  syslog(LOG_EMERG, fmt, ap);
#else
  subsyslog(LOG_FATAL, fmt, ap);
#endif	/* USE_SYSLOG */
  va_end(ap);

  if (httpd->pid > 0) {
    (void)unlink(httpd->pidFile);	/* ignore error */
  }

  exit(1);
}

#ifndef	USE_SYSLOG
/*
 * Function Name:	wvSyslog
 * Description:		write log messages
 * Arguments:		log_t, log messages
 * Return Value:	none
 */
void
#if	__STDC__
wvSyslog(log_t type, const char *message, ...)
#else
wvSyslog(type, message, va_alist)
	log_t	type;
	char *	message;
	va_dcl
#endif	/* __STDC__ */
{
  va_list	ap;
#if	__STDC__
  va_start(ap, message);
#else
  va_start(ap);
#endif	/* __STDC__ */
  subsyslog(type, message, ap);
  va_end(ap);
}

static void
subsyslog(type, fmt, ap)
	log_t			type;
	register const char *	fmt;
	va_list			ap;
{
  fprintf(httpd->errorLog, "[%s] ", getCurrentTime);
  switch (type) {
    case LOG_EMERG:	/* system is unusable */
      fputs("[emerg]   ", httpd->errorLog);
      break;
    case LOG_ALERT:	/* action must be taken immediately */
      fputs("[alert]   ", httpd->errorLog);
      break;
    case LOG_CRIT:	/* critical conditions */
      fputs("[crit]    ", httpd->errorLog);
      break;
    case LOG_ERR:	/* error conditions */
      fputs("[error]   ", httpd->errorLog);
      break;
    case LOG_WARNING:	/* warning conditions */
      fputs("[warning] ", httpd->errorLog);
      break;
    case LOG_NOTICE:	/* normal but significant condition */
      fputs("[notice]  ", httpd->errorLog);
      break;
    case LOG_INFO:	/* informational */
      fputs("[info]    ", httpd->errorLog);
      break;
    case LOG_DEBUG:	/* debug-level messages */
      fputs("[debug]   ", httpd->errorLog);
      break;
    default:
      break;
  }
  (void)vfprintf(httpd->errorLog, fmt, ap);
}
#endif	/* USE_SYSLOG */

