/*****************************************************************************
 *
 * FILE:	module.c
 * DESCRIPTION:	module manager for wyvern
 * DATE:	Wed, Sep 13 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 2000-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: module.c,v 1.35.2.1 2002/01/30 13:26:33 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#if	HAVE_DLOPEN
#include <dlfcn.h>
#else
#if	HAVE_SHL_LOAD
#include <dl.h>
#else
#if	HAVE_LIBDLD
#include <dld.h>
#endif	/* HAVE_LIBDLD */
#endif	/* HAVE_SHL_LOAD */
#endif	/* HAVE_DLOPEN */
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#define	DLOPEN_MODE		(RTLD_LAZY)


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void *	load(String);
static void	unload(void *);
static void *	getProc(void *, String);


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	initPluggableModules
 * Description:		initialize module structure
 * Arguments:		none
 * Return Value:	none
 */
void
initPluggableModules(void)
{
  /* log */
  httpd->mod.log.handle		= NULL;
  httpd->mod.log.write		= writeLogs;
  /* ident */
  httpd->mod.ident.handle	= NULL;
  httpd->mod.ident.set		= NULL;
  /* access */
  httpd->mod.access.handle	= NULL;
  httpd->mod.access.check	= NULL;
  /* auth */
  httpd->mod.auth.handle	= NULL;
  httpd->mod.auth.allow		= NULL;
  /* dir */
  httpd->mod.dir.handle		= NULL;
  httpd->mod.dir.listing	= NULL;
  /* cgi */
  httpd->mod.cgi.handle		= NULL;
  httpd->mod.cgi.run		= NULL;
  /* ssi */
  httpd->mod.ssi.handle		= NULL;
  httpd->mod.ssi.run		= NULL;
#if	HAVE_ZLIB_H
  /* gzip */
  httpd->mod.gzip.handle	= NULL;
  httpd->mod.gzip.gzcompress	= NULL;
#endif	/* HAVE_ZLIB_H */
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	loadModule
 * Description:		load a module
 * Arguments:		module name, module file
 * Return Value:	none
 */
void
loadModule(moduleName, moduleFile)
	String	moduleName;
	String	moduleFile;
{
  if (strcasecmp(moduleName, MOD_LOG_NAME) == 0) {
    httpd->mod.log.handle = load(moduleFile);
    httpd->mod.log.write  =
	(void (*)(Thread *))getProc(httpd->mod.log.handle, "writeMoreLogs");
    if (httpd->mod.log.write == NULL) {
      httpd->mod.log.write  = writeLogs;
    }
  }
  else if (strcasecmp(moduleName, MOD_IDENT_NAME) == 0) {
    httpd->mod.ident.handle = load(moduleFile);
    httpd->mod.ident.set    =
	(void (*)(Thread *))getProc(httpd->mod.ident.handle, "useRemoteIdent");
  }
  else if (strcasecmp(moduleName, MOD_ACCESS_NAME) == 0) {
    httpd->mod.access.handle = load(moduleFile);
    httpd->mod.access.check  =
	(bool_t (*)(Thread *))getProc(httpd->mod.access.handle, "allowHost");
  }
  else if (strcasecmp(moduleName, MOD_AUTH_NAME) == 0) {
    httpd->mod.auth.handle = load(moduleFile);
    httpd->mod.auth.allow  =
	(bool_t (*)(Thread *))getProc(httpd->mod.auth.handle, "allowUser");
  }
  else if (strcasecmp(moduleName, MOD_DIR_NAME) == 0) {
    httpd->mod.dir.handle  = load(moduleFile);
    httpd->mod.dir.listing =
	(void (*)(Thread *))getProc(httpd->mod.dir.handle, "directoryListing");
  }
  else if (strcasecmp(moduleName, MOD_CGI_NAME) == 0) {
    httpd->mod.cgi.handle = load(moduleFile);
    httpd->mod.cgi.run    =
	(void (*)(Thread *))getProc(httpd->mod.cgi.handle, "runScript");
  }
  else if (strcasecmp(moduleName, MOD_SSI_NAME) == 0) {
    httpd->mod.ssi.handle = load(moduleFile);
    httpd->mod.ssi.run    =
	(void (*)(Thread *))getProc(httpd->mod.ssi.handle, "parseSSI");
  }
#if	HAVE_ZLIB_H
  else if (strcasecmp(moduleName, MOD_GZIP_NAME) == 0) {
    httpd->mod.gzip.handle	= load(moduleFile);
    httpd->mod.gzip.gzcompress	=
	(void (*)(Thread *))getProc(httpd->mod.gzip.handle, "gzipCompress");
  }
#endif	/* HAVE_ZLIB_H */
  else {	/* invalid module name */
    wvSyslog(LOG_ERR, "invalid module name: %s\n",  moduleName);
    return;
  }
}


/*
 * Function Name:	cleanupModules
 * Description:		unload all modules
 * Arguments:		none
 * Return Value:	none
 */
void
cleanupModules(void)
{
  /* unload Log module */
  if (httpd->mod.log.handle != NULL) {
    unload(httpd->mod.log.handle);
  }
  /* unload Identification Protocol (RFC1413) module */
  if (httpd->mod.ident.handle != NULL) {
    unload(httpd->mod.ident.handle);
  }
  /* unload Access module */
  if (httpd->mod.access.handle != NULL) {
    unload(httpd->mod.access.handle);
  }
  /* unload Basic/Digest Authentication module */
  if (httpd->mod.auth.handle != NULL) {
    unload(httpd->mod.auth.handle);
  }
  /* unload Directory Listing module */
  if (httpd->mod.dir.handle != NULL) {
    unload(httpd->mod.dir.handle);
  }
  /* unload CGI module */
  if (httpd->mod.cgi.handle != NULL) {
    unload(httpd->mod.cgi.handle);
  }
  /* unload SSI module */
  if (httpd->mod.ssi.handle != NULL) {
    unload(httpd->mod.ssi.handle);
  }
#if	HAVE_ZLIB_H
  /* unload gzip module */
  if (httpd->mod.gzip.handle != NULL) {
    unload(httpd->mod.gzip.handle);
  }
#endif	/* HAVE_ZLIB_H */

  initPluggableModules();
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	load
 * Description:		load module dynamically
 * Arguments:		module file
 * Return Value:	successful: handle, otherwise: NULL
 */
static void *
load(moduleFile)
	String	moduleFile;
{
#if	HAVE_DLOPEN
  void *	handle;
#elif	HAVE_SHL_LOAD
  shl_t		handle;
#elif	HAVE_LIBDLD
#endif	/* HAVE_DLOPEN */

#if	HAVE_DLOPEN
  handle = dlopen(moduleFile, DLOPEN_MODE);
#elif	HAVE_SHL_LOAD
  handle = shl_load(moduleFile, BIND_IMMEDIATE || BIND_FIRST ||
				BIND_TOGETHER || BIND_VERBOSE ||
				DYNAMIC_PATH, 0L);
#elif	HAVE_LIBDLD
#endif	/* HAVE_DLOPEN */
  if (handle == NULL) {
    wvSyslog(LOG_ERR, "failed to load a module (%s) --- %s\n",
	     moduleFile, (String)dlerror());
  }
  return handle;
}

/*
 * Function Name:	unload
 * Description:		unload module
 * Arguments:		module handler
 * Return Value:	none
 */
static void
unload(handle)
	void *	handle;
{
#if	HAVE_DLOPEN
  int	ret = dlclose(handle);
#elif	HAVE_SHL_LOAD
  int	ret = shl_unload((shl_t)handle);
#elif	HAVE_LIBDLD
#endif	/* HAVE_DLOPEN */
  if (ret < 0) {
    wvSyslog(LOG_ERR, "failed to unload a module --- %s\n",
	     (String)dlerror());
  }
}

/*
 * Function Name:	getProc
 * Description:		get function address from loaded module
 * Arguments:		module handler, function name
 * Return Value:	function address
 */
static void *
getProc(handle, funcName)
	void *	handle;
	String	funcName;
{
  void *	fptr;
#if	HAVE_DLOPEN
#if defined(USCORE) && !defined(DLSYM_ADDS_USCORE)
  String	usymb;
  size_t	len;

  len	= strlen(funcName);
  usymb	= (String)xcalloc(len + 2, sizeof(char));
  if (usymb == NULL) {
    wvSyslog(LOG_ERR, "failed to get a symbol from a module\n");
    return NULL;
  }
  *usymb = '_';
  strncpy(usymb + 1, funcName, len);
  fptr = dlsym(handle, usymb);
  XFree(usymb);
#else
  fptr = dlsym(handle, funcName);
#endif
#elif	HAVE_SHL_LOAD
  int	status;

  status = shl_findsym((shl_t*)&handle, funcName, TYPE_PROCEDURE, (void*)&fptr);
#elif	HAVE_LIBDLD
#endif	/* HAVE_DLOPEN */
  if (fptr == NULL) {
    wvSyslog(LOG_ERR, "failed to get a symbol from a module\n");
  }

  return fptr;
}

