/*****************************************************************************
 *
 * FILE:	fform.c
 * DESCRIPTION:	library for file form of directory listing module
 * DATE:	Tue, May 15 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2001-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 2001-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: fform.c,v 1.6 2002/01/12 12:27:46 kouichi Exp $
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "wyvern.h"
#include "fform.h"


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	printFileForm
 * Description:		print the file form
 * Arguments:		file name, file mode, file pointer
 * Return Value:	none
 */
void
printFileForm(fname, fmode, fout)
	String		fname;
	mode_t		fmode;
	register FILE *	fout;
{
  fputs("<img src=\"/icons/", fout);

  if (S_ISDIR(fmode)) {
    fputs("dir.gif\" alt=\"[DIR]\" ", fout);
  }
  else if (S_ISREG(fmode)) {
/*
 * Uncomment the following line if you don't like the detailed icons.
 */
/*#define	NO_DETAIL_INDEXING	1*/
#ifndef	NO_DETAIL_INDEXING
    String	p = NULL;

    p = strrchr(fname, '.');
    if (p == NULL) {
      fputs("unknown.gif\" alt=\"[   ]\" ", fout);
    }
    else {
      const struct fileForm *	ff = getFileForm(p, strlen(p));

      if (ff == NULL) {
	fputs("generic.gif\" alt=\"[   ]\" ", fout);
      }
      else {
	switch (ff->form) {
	  case FF_COMPRESSED:	/* compressed */
	    fputs("compressed.gif\" alt=\"[CMP]\" ", fout);
	    break;
	  case FF_TAR:		/* tar archived */
	    fputs("tar.gif\" alt=\"[TAR]\" ", fout);
	    break;
	  case FF_ARCHIVE:	/* archived */
	    fputs("compressed.gif\" alt=\"[ARC]\" ", fout);
	    break;
	  case FF_BINARY:		/* binary */
	    fputs("binary.gif\" alt=\"[BIN]\" ", fout);
	    break;
	  case FF_HTML:		/* HTML */
	    fputs("layout.gif\" alt=\"[HTM]\" ", fout);
	    break;
	  case FF_TEXT:		/* text */
	    fputs("text.gif\" alt=\"[TXT]\" ", fout);
	    break;
	  case FF_PDF:		/* PDF */
	    fputs("pdf.gif\" alt=\"[PDF]\" ", fout);
	    break;
	  case FF_DVI:		/* DVI */
	    fputs("dvi.gif\" alt=\"[DVI]\" ", fout);
	    break;
	  case FF_POSTSCRIPT:	/* PostScript */
	    fputs("ps.gif\" alt=\"[PS ]\" ", fout);
	    break;
	  case FF_C:		/* C language */
	    fputs("c.gif\" alt=\"[C  ]\" ", fout);
	    break;
	  case FF_F77:		/* Fortran language */
	    fputs("f.gif\" alt=\"[F77]\" ", fout);
	    break;
	  case FF_PERL:		/* Perl language */
	    fputs("p.gif\" alt=\"[P  ]\" ", fout);
	    break;
	  case FF_TEX:		/* TeX/LaTeX */
	    fputs("tex.gif\" alt=\"[TEX]\" ", fout);
	    break;
	  case FF_PATCH:		/* patch file */
	    fputs("patch.gif\" alt=\"[PAT]\" ", fout);
	    break;
	  case FF_SHELL:		/* shell script */
	    fputs("script.gif\" alt=\"[SH ]\" ", fout);
	    break;
	  case FF_IMAGE:		/* image */
	    fputs("image1.gif\" alt=\"[IMG]\" ", fout);
	    break;
	  case FF_MOVIE:		/* movie */
	    fputs("movie.gif\" alt=\"[MOV]\" ", fout);
	    break;
	  case FF_SOUND:		/* sound/audio */
	    fputs("sound2.gif\" alt=\"[SND]\" ", fout);
	    break;
	  default:
	    break;
	}
      }
    }
#else
    fputs("generic.gif\" alt=\"[   ]\" ", fout);
#endif	/* NO_DETAIL_INDEXING */
  }
  else if (S_ISLNK(fmode)) {
    fputs("link.gif\" alt=\"[LNK]\" ", fout);
  }
  else {
    fputs("unknown.gif\" alt=\"[UNK]\" ", fout);
  }

  fputs("width=\"20\" height=\"22\">", fout);
}

/*
 * Function Name:	printFileDescription
 * Description:		print the file description in HTML text
 * Arguments:		file name, file pointer
 * Return Value:	none
 */
void
printFileDescription(fname, fout)
	String		fname;
	register FILE *	fout;
{
#ifndef	NO_DETAIL_INDEXING
  String	p = NULL;

  p = strrchr(fname, '.');
  if (p == NULL) {
    fputs("File", fout);
  }
  else if (strcmp(p, ".htm") == 0 || strcmp(p, ".html") == 0) {
    fputs("HTML document file", fout);
  }
  else if (strcmp(p, ".shtml") == 0) {
    fputs("SSI HTML document file", fout);
  }
  else if (strcmp(p, ".cgi") == 0) {
    fputs("CGI program", fout);
  }
  else if (strcmp(p, ".txt") == 0) {
    fputs("TEXT document file", fout);
  }
  else if (strcmp(p, ".ps") == 0) {
    fputs("Postscript document file", fout);
  }
  else if (strcmp(p, ".pdf") == 0) {
    fputs("PDF document file", fout);
  }
  else if (strcmp(p, ".tgz") == 0) {
    fputs("GZIP compressed tar archive", fout);
  }
  else if (strcmp(p, ".tar") == 0) {
    fputs("TAR archive", fout);
  }
  else if (strcmp(p, ".zip") == 0) {
    fputs("ZIP archive", fout);
  }
  else if (strcmp(p, ".lzh") == 0) {
    fputs("LHA archive", fout);
  }
  else {
    bool_t	f_tar = FALSE;

    if (strncmp(p - 4, ".tar", 4) == 0) {
      f_tar = TRUE;
    }

    if (strcmp(p, ".Z") == 0) {
      fputs("UNIX compressed", fout);
    }
    else if (strcmp(p, ".gz") == 0) {
      fputs("GZIP compressed", fout);
    }
    else if (strcmp(p, ".bz") == 0) {
      fputs("BZIP compressed", fout);
    }
    else if (strcmp(p, ".bz2") == 0) {
      fputs("BZIP2 compressed", fout);
    }
    else {
      fputs("File", fout);
      return;
    }

    if (f_tar) {
      fputs(" tar archive", fout);
    }
    else {
      fputs(" file", fout);
    }
  }
#else
  fputs("File", fout);
#endif	/* NO_DETAIL_INDEXING */
}

