/*****************************************************************************
 *
 * FILE:	mod_log.c
 * DESCRIPTION:	wyvern logging module
 * DATE:	Fri, Sep 15 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 2000-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_log.c,v 1.10.2.2 2002/01/27 16:10:30 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include "wyvern.h"


/*****************************************************************************
 *
 *	declaration of global functions
 *
 *****************************************************************************/
extern void	writeLogs(Thread *);


/*****************************************************************************
 *
 *	definition of macros
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

#if	DEBUG
void
_init(void)
{
  fputs("[mod_log]: plugged-in\n", stderr);
}
#endif	/* DEBUG */

/*
 * Function Name:	writeLogs
 * Description:		write the log message in log file
 * Arguments:		thread
 * Return Value:	none
 */
void
writeLogs(thd)
	register Thread *	thd;
{
  switch (thd->rstat.class) {
    case CLASS_INFORMATIONAL:	/* Informational 1xx	*/
    case CLASS_SUCCESSFUL:	/* Successful 2xx	*/
    case CLASS_REDIRECTION:	/* Redirection 3xx	*/
      if (thd->request.x_forwarded_for != NULL) {
	fprintf(httpd->accessLog,
		"%s(%s) ",
		thd->client.name == NULL ? thd->client.addr : thd->client.name,
		thd->request.x_forwarded_for);

      }
      else {
	fprintf(httpd->accessLog, "%s ",
		thd->client.name == NULL ? thd->client.addr : thd->client.name);
      }
      fprintf(httpd->accessLog, "%s %s [%s] \"%s %s %s\" %d ",
		thd->identUser == NULL ? "-" : thd->identUser,
		thd->authUser == NULL ? "-" : thd->authUser,
		getTimeOfRFC822(thd->lastAccessTime),
		thd->method.name,
		thd->requestURI,
		thd->http.protocol,
		thd->rstat.code);
      if (thd->rstat.class == CLASS_REDIRECTION) {
	fputs("-\n", httpd->accessLog);
      }
      else {
	if (thd->method.type == M_HEAD) {
	  fprintf(httpd->accessLog, "%ld\n", (long)thd->file.st_size);
	}
	else if (thd->method.type == M_GET || thd->method.type == M_POST) {
	  fprintf(httpd->accessLog, "%d\n", thd->request.content_length);
	}
	else {
	  fputs("-\n", httpd->accessLog);
	}
      }
      break;
    case CLASS_CLIENT_ERROR:	/* Client Error 4xx	*/
    case CLASS_SERVER_ERROR:	/* Server Error 5xx	*/
    default:
      if (thd->request.x_forwarded_for != NULL &&
	  strncasecmp(thd->request.x_forwarded_for, "unknown", 7) == 0) {
	wvSyslog(LOG_WARNING,
		 "(%s) \"%s %s %s\" "
		 "request forwarded from unknown host.\n",
		 thd->client.name == NULL ? thd->client.addr : thd->client.name,
		 thd->method.name,
		 thd->requestURI,
		 thd->http.protocol);
      }
      else if (thd->method.name == NULL || thd->requestURI[0] == EOL) {
	wvSyslog(LOG_EMERG, "attacked or scanned from %s; %s (%d)\n",
		 thd->client.name == NULL ? thd->client.ip : thd->client.name,
		 getResponsePhrase(thd->rsCode),
		 getResponseCode(thd->rsCode));
      }
      else {
	wvSyslog(LOG_ERR, "%s \"%s %s %s\" %s (%d)\n",
		 thd->client.name == NULL ? thd->client.addr : thd->client.name,
		 thd->method.name,
		 thd->requestURI,
		 thd->http.protocol,
		 thd->rstat.phrase,
		 thd->rstat.code);
      }
      break;
  }
}

