/*****************************************************************************
 *
 * FILE:	network.c
 * DESCRIPTION:	lookup the client host
 * DATE:	Sun, Sep 26 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: network.c,v 1.35.2.3 2002/01/30 13:26:33 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#if	HAVE_PATHS_H
#include <paths.h>
#endif	/* HAVE_PATHS_H */
#include <errno.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static String	getFQDN(void);


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	openServiceChannel
 * Description:		initialize server
 * Arguments:		port
 * Return Value:	none
 */
void
openServiceChannel(port)
	unsigned int	port;
{
#if	USE_IPV6
  struct sockaddr_in6	serv;
#else
  struct sockaddr_in	serv;
#endif	/* USE_IPV6 */
  int			flags;
  int			on;

  /* create the socket */
#if	USE_IPV6
  httpd->socket.soc = socket(PF_INET6, SOCK_STREAM, 0);
#else
  httpd->socket.soc = socket(PF_INET, SOCK_STREAM, 0);
#endif	/* USE_IPV6 */
  if (httpd->socket.soc == -1) {
    die("failed to create a socket\n");
  }

  /* nonblocking socket */
  flags = fcntl(httpd->socket.soc, F_GETFL, 0);
  if (flags != -1) {
    flags |= O_NONBLOCK;
    if (fcntl(httpd->socket.soc, F_SETFL, flags) == -1) {
      die("failed to change non-blocking mode\n");
    }
  }

  /* allow reuse of local addresses */
  on = 1;
  if (setsockopt(httpd->socket.soc, SOL_SOCKET, SO_REUSEADDR, (void *)&on,
	sizeof(on)) == -1) {
    die("failed to set a socket option\n");
  }
#if	defined(DEBUG) || defined(SO_REUSEPORT)
  if (setsockopt(httpd->socket.soc, SOL_SOCKET, SO_REUSEPORT, (void *)&on,
	sizeof(on)) == -1) {
    die("failed to set a socket option\n");
  }
#endif	/* DEBUG || SO_REUSEPORT */

  /* overwrite the port number by option '-p' */
  httpd->port = port ? port : httpd->port;

  /* initialize socket address */
  memset(&serv, 0, sizeof(serv));
#if	USE_IPV6
  serv.sin6_family	= PF_INET6;
  serv.sin6_port	= htons(httpd->port);
  if (httpd->bindAddress != NULL) {
    if (inet_pton(AF_INET6, httpd->bindAddress, &serv.sin6_addr) != 1) {
      die("BindAddress[%s] is not valid.\n", httpd->bindAddress);
    }
  }
  else {
    serv.sin6_addr	= in6addr_any;
  }
#else
  serv.sin_family	= PF_INET;
  serv.sin_port		= htons(httpd->port);
  if (httpd->bindAddress != NULL) {
    if (inet_pton(AF_INET, httpd->bindAddress, &serv.sin_addr) != 1) {
      die("BindAddress[%s] is not valid.\n", httpd->bindAddress);
    }
  }
  else {
    serv.sin_addr.s_addr= htonl(INADDR_ANY);
  }
#endif	/* USE_IPV6 */

  /* bind the socket */
  if (bind(httpd->socket.soc, (struct sockaddr *)&serv, sizeof(serv)) == -1) {
    die("failed to bind a name to a socket\n");
  }

  /* listen for connection on the socket */
  if (listen(httpd->socket.soc, httpd->maxRequest) == -1) {
    die("failed to listen for connection on a socket\n");
  }

  /* set socket address of wyvern */
  httpd->socket.addr = serv;	/* used in ident.mod */
}

/*
 * Function Name:	xclose
 * Description:		close a file/socket descriptor
 * Arguments:		descriptor
 * Return Value:	same as close(2)
 */
int
xclose(d)
	int	d;
{
  int	ret;

  while (((ret = close(d)) < 0) && (errno == EINTR)) {
    ;
  }
  return ret;
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	getFQDN
 * Description:		get Full-Qualified Domain Name
 * Arguments:		none
 * Return Value:	successful: FQDN hostname, otherwise: NULL
 */
static String
getFQDN(void)
{
  struct hostent *	hp;
  char			name[MAX_HOSTNAME_LEN];

  memset(name, 0, sizeof(name));
  if (gethostname(name, sizeof(name) - 1) != 0) {
    return NULL;	/* failed to get hostname */
  }

  hp = gethostbyname(name);
  if (hp == NULL) {
    return NULL;	/* failed to get hostname */
  }

  if (strchr(hp->h_name, '.') == NULL) {/* not include '.' in hostname */
    if (hp->h_aliases != NULL) {	/* has an alias list */
      int	i;
      for (i = 0; hp->h_aliases[i] != NULL; i++) {
	if (hp->h_aliases[i] != NULL &&
	    strchr(hp->h_aliases[i], '.') != NULL &&
	    (strncasecmp(hp->h_aliases[i], hp->h_name,
			 strlen(hp->h_name)) == 0)) {
	  return xstrdup(hp->h_aliases[i]);
	}
      }
    }
  }

#if	0
  if (hp->h_addr_list != NULL && hp->h_addr_list[0]) {
    return xstrdup(hp->h_addr_list[0]);
  }
#endif	/* NOUSE */

  return xstrdup(name);
}

/*
 * Function Name:	getServerName
 * Description:		get server name
 * Arguments:		none
 * Return Value:	server name
 */
String
getServerName(void)
{
  String	name;

  name = getFQDN();
  if (name != NULL) {
    return name;
  }
  else {	/* failed to get hostname, fall back to the loopback address */
    return xstrdup("127.0.0.1");
  }
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	setClientName
 * Description:		set a connecting client's hostname & IP address
 * Arguments:		thread, client
 * Return Value:	none
 */
void
setClientName(thd, cli)
	register Thread *		thd;
#if	USE_IPV6
	struct sockaddr_storage *	cli;
#else
	struct sockaddr_in *		cli;
#endif	/* USE_IPV6 */
{
#if	USE_IPV6
  struct sockaddr_in6 *	sin6 = (struct sockaddr_in6 *)cli;

  /* set client IP address */
  thd->client.ip = (String)xmalloc(INET6_ADDRSTRLEN + 1);
  inet_ntop(sin6->sin6_family, &sin6->sin6_addr, thd->client.ip,
	    INET6_ADDRSTRLEN);

  /* set remote port */
  thd->client.port = ntohs(sin6->sin6_port);	/* correct? */

  /* store sockaddr_in structure for ident module */
  thd->client.addr = *sin6;
#else
  /* set client IP address */
  xfree((String)thd->client.ip);
  thd->client.ip = xstrdup(inet_ntoa(cli->sin_addr));

  /* set remote port */
  thd->client.port = ntohs(cli->sin_port);

  /* store sockaddr_in structure for ident module */
  thd->client.addr = *cli;
#endif /* USE_IPV6 */
  XFree(thd->client.name);
}


/*****************************************************************************
 *****************************************************************************/

#if	MAIL_ANNOUNCE
/*
 * Function Name:	sendMail
 * Description:		send an error mail to the server admin
 * Arguments:		none
 * Return Value:	none
 */
void
sendMail(void)
{
#define	SENDMAIL_OPTIONS	" -t -oi"
  register FILE *	fout;
  String		cmd;

  if (xasprintf(&cmd, "%s -t -oi", _PATH_SENDMAIL) == -1) {
    wvSyslog(LOG_ERR, "failed to send a mail\n");
    return;
  }

  fout = popen(cmd, "w");
  xfree((String)cmd);
  if (fout == NULL) {
    wvSyslog(LOG_ERR, "failed to send a mail\n");
    return;
  }

  /* put a mail header */
  fprintf(fout,
	  "To: %s\n"
	  "Subject: Wyvern Report\n\n",
	  httpd->vhost->serverAdmin);

  /* put a mail body */
  fprintf(fout,
	  "Please restart Wyvern because it's terminated by signal.\n"
	  "Thank you.\n"
	  "---\n"
	  "%s\n",
	  SERVER_SOFTWARE);

  pclose(fout);
}
#endif	/* MAIL_ANNOUNCE */

