/*****************************************************************************
 *
 * FILE:	response.c
 * DESCRIPTION:	response the data to the client
 * DATE:	Sun, Oct  3 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: response.c,v 1.68.2.4 2002/01/30 04:32:44 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <errno.h>
#include <time.h>
#if	!defined(USE_SSL) && defined(USE_CONTENT_MD5)
#include "md5.h"
#endif	/* USE_SSL, USE_CONTENT_MD5 */
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#define	TIME_LIMIT	5	/* send (TIME_LIMIT * RESPONSE_BUFSIZE) bytes */
				/* at once */


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	sendStatusLine(Thread *);
static void	sendHeader(Thread *);
static void	sendEntity(Thread *);
static void	sendEntityDone(Thread *);
#if	USE_CONTENT_MD5
static void	setMD5(Thread *);
#endif	/* USE_CONTENT_MD5 */
#if	0
static void	setTransferEncoding(Thread *);
#endif
static void	setContentLocation(Thread *);


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	sendResponse
 * Description:		send the response to a client
 * Arguments:		thread
 * Return Value:	none
 */
void
sendResponse(thd)
	register Thread *	thd;
{
  /* send status line */
  sendStatusLine(thd);

  /* send header fields and entity */
  switch (thd->method.type) {
    case M_HEAD:
      sendHeader(thd);
      sendEntityDone(thd);
      break;
    case M_GET:
    case M_POST:
      sendHeader(thd);
      sendEntity(thd);
      break;
    case M_PUT:
    case M_DELETE:
    case M_MOVE:
    case M_OPTIONS:
    case M_TRACE:
    case M_CONNECT:
    case M_SEARCH:
    default:
      sendHeader(thd);
      sendEntityDone(thd);
      break;
  }
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	sendStatusLine
 * Description:		send HTTP status line for request
 * Arguments:		thread
 * Return Value:	none
 */
static void
sendStatusLine(thd)
	register Thread *	thd;
{
  if (thd->cgi.status != NULL) {
    iosPrintf(thd, "%s %s", thd->http.protocol, thd->cgi.status);
  }
  else {
    iosPrintf(thd, "%s %d %s" CRLF,
		   thd->http.protocol, getResponseCode(thd->rsCode),
		   getResponsePhrase(thd->rsCode));
  }
}

/*
 * Function Name:	sendHeader
 * Description:		send the response header to a client
 * Arguments:		thread
 * Return Value:	none
 */
static void
sendHeader(thd)
	register Thread *	thd;
{
  /* send standard response header fields */
  thd->response.date  = xstrdup(getCurrentTime);
  thd->response.allow = xstrdup(WYVERN_METHODS);
  iosPrintf(thd, "Date: %s" CRLF
		 "Server: %s" CRLF
		 "Allow: %s" CRLF,
		 thd->response.date,
		 SERVER_VERSION,
		 thd->response.allow);
 
  /* send a response header field for Authenticate */
  if (thd->rsCode == CODE_UNAUTHORIZED) {
    iosPrintf(thd, "WWW-Authenticate: %s" CRLF, thd->response.www_authenticate);
  }

#if	HAVE_ZLIB_H
  /* encoding content with gzip */
  if (thd->state != IN_ERROR && httpd->mod.gzip.gzcompress != NULL) {
    httpd->mod.gzip.gzcompress(thd);
    if (thd->response.content_encoding != NULL) {
      iosPrintf(thd, "Content-Encoding: %s" CRLF,
		thd->response.content_encoding);
    }
  }
#endif	/* HAVE_ZLIB_H */

  /* send a response header field for Redirect */
  if (thd->rsCode == CODE_MOVED_PERMANENTLY) {
    iosPrintf(thd, "Location: %s" CRLF, thd->response.location);
    return;
  }
  /* send a response header field for precondition request */
  else if (thd->rsCode == CODE_NOT_MODIFIED) {
    setContentLocation(thd);
    iosPrintf(thd, "Content-Location: %s" CRLF,
		   thd->response.content_location);
    return;
  }
  /* send HTTP header fields in case with no entity */
  else if (thd->ctype == C_MSG) {
    iosPuts(thd, "Content-Type: text/html" CRLF
		 "Connection: close" CRLF
		 CRLF);
    thd->keepAlive.state = KA_CLOSE;
    return;
  }
  /* send no more headers in case of CGI response */
  else if (thd->ctype == C_CGI) {
    if (thd->response.content_type != NULL) {
      iosPrintf(thd, "Content-Type: %s" CRLF, thd->response.content_type);
    }
    if (thd->response.content_length > 0) {
      iosPrintf(thd, "Content-Length: %d" CRLF, thd->response.content_length);
    }
    if (thd->cgi.headers != NULL) {
      register StrList *	slp;
      for (slp = thd->cgi.headers; slp != NULL; slp = slp->next) {
	iosPuts(thd, slp->text);
      }
    }
  }
  /* send other entity header fields */
  else {
    thd->response.content_type  = xstrdup(thd->mtype);
    thd->response.accept_ranges = xstrdup(RANGE_UNIT);
    iosPrintf(thd, "Content-Type: %s" CRLF
		   "Accept-Range: %s" CRLF,
		   thd->response.content_type,
		   thd->response.accept_ranges);

    /* partial content */
    if (thd->ctype == C_PAR) {
      thd->response.content_length = thd->range.last - thd->range.first + 1;
      iosPrintf(thd, "Content-Range: %s" CRLF, thd->response.content_range);
    }
    /* other content */
    else {
      thd->response.content_length = thd->file.st_size;
    }
    thd->response.last_modified = xstrdup(getTimeOfRFC822(thd->file.st_mtime));
    iosPrintf(thd, "Content-Length: %d" CRLF
		   "Last-Modified: %s" CRLF
		   "ETag: %s" CRLF,
		   thd->response.content_length,
		   thd->response.last_modified,
		   thd->response.etag);
#if	DEBUG
    fprintf(stderr, "[response] Content-Type: %s\n", thd->mtype);
    fprintf(stderr, "[response] Content-Length: %d\n",
		    thd->response.content_length);
#endif	/* DEBUG */
#if	USE_CONTENT_MD5
    setMD5(thd);
    iosPrintf(thd, "Content-MD5: %s" CRLF, thd->response.content_md5);
#endif	/* USE_CONTENT_MD5 */
  }

  /** send HTTP header fields for Keep-Alive mode */
  if (thd->keepAlive.state == KA_CONTINUE && thd->keepAlive.max > 0) {
#if	BUGGY_KEEP_ALIVE
    if (thd->http.version == HTTP_10) {
      iosPrintf(thd, "Connection: Keep-Alive" CRLF
		     "Keep-Alive: timeout=%d, max=%d" CRLF,
		     thd->keepAlive.timeout, thd->keepAlive.max);
    }
#endif	/* BUGGY_KEEP_ALIVE */
  }
  else {
    iosPuts(thd, "Connection: close" CRLF);
    thd->keepAlive.state = KA_CLOSE;
  }
#if	DEBUG
  fprintf(stderr, "[Keep-Alive(%d)]: status=%d, timeout=%d, max=%d, %s\n",
		  thd->client.port, thd->keepAlive.state,
		  thd->keepAlive.timeout, thd->keepAlive.max, thd->requestURI);
#endif	/* DEBUG */

  iosPuts(thd, CRLF);
#if	0
  /*
   *	send the HTTP header
   */
  iosPrintf(thd, "Age: %s" CRLF
		 "Content-Language: %s" CRLF
		 "Expires: %s" CRLF,
		 thd->response.age,
		 thd->response.content_language,
		 thd->response.expires);
#endif
}

/*
 * Function Name:	sendEntity
 * Description:		send the response entity to a client
 * Arguments:		thread
 * Return Value:	none
 */
static void
sendEntity(thd)
	register Thread *	thd;
{
  if (thd->state == IN_ERROR) {	/* send the entity of error	*/
    sendErrorMessage(thd);
    sendEntityDone(thd);
  }
  else if (thd->ctype == C_MSG) {
    /* send no entity! */
    sendEntityDone(thd);
  }
  else if (thd->ctype == C_PAR) {
    thd->parts.fsize	= thd->response.content_length;
    thd->parts.bsize	= MIN(thd->parts.fsize, RESPONSE_BUFSIZE);
    thd->parts.sum	= 0;
    sendingEntity(thd);
  }
  else {
    sendingEntity(thd);
  }
}

/*
 * Function Name:	sendingEntity
 * Description:		send the response entity to a client
 * Arguments:		thread
 * Return Value:	none
 */
void
sendingEntity(thd)
	register Thread *	thd;
{
  int		c = 0;
  char		buf[BUF_SIZE];
  size_t	size;

  /* partial content */
  if (thd->ctype == C_PAR) {
    /* at most, send data which size is (TIME_LIMIT * thd->parts.bsize) bytes */
    while (!feof(thd->fp)) {
      memset(buf, 0, sizeof(buf));
      size = fread(buf, 1, (size_t)thd->parts.bsize, thd->fp);
      thd->parts.sum  += size;
      thd->parts.bsize = MIN(thd->parts.bsize,
			     thd->parts.fsize - thd->parts.sum);
      iosWrite(thd, buf, size);
      if (thd->parts.sum >= thd->parts.fsize) {
	break;
      }

      if (c++ > TIME_LIMIT) {
	thd->state = NOW_SENDING;
	return;
      }
    }
  }
  /* standard content */
  else {
    /* at most, send data which size is (TIME_LIMIT * RESPONSE_BUFSIZE) bytes */
    while (!feof(thd->fp)) {
      memset(buf, 0, sizeof(buf));
      size = fread(buf, 1, (size_t)sizeof(buf), thd->fp);
      iosWrite(thd, buf, size);
      if (c++ > TIME_LIMIT) {
	thd->state = NOW_SENDING;
	return;
      }
    }
  }

  sendEntityDone(thd);
}

/*
 * Function Name:	sendEntityDone
 * Description:		finish to send the response entity to a client
 * Arguments:		thread
 * Return Value:	none
 */
static void
sendEntityDone(thd)
	register Thread *	thd;
{
  iosFlush(thd);
  thd->state = TERMINATED;
#if	DEBUG
  if (thd->keepAlive.state == KA_CONTINUE) {
    fprintf(stderr, "[done(%d)]: keep-alive continued...\n", thd->client.port);
  }
  else {
    fprintf(stderr, "[done(%d)]: sending entity ... done\n", thd->client.port);
  }
#endif	/* DEBUG */
  httpd->mod.log.write(thd);	/* write logs */
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	setETag
 * Description:		set the entity tag
 * Arguments:		thread
 * Return Value:	none
 */
void
setETag(thd)
	register Thread *	thd;
{
  if (xasprintf(&thd->response.etag, "\"%lx-%lx-%lx\"",
		(long)thd->file.st_ino,
		(long)thd->file.st_size,
		thd->file.st_mtime) == -1) {
    wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
  }
}

#if	USE_CONTENT_MD5
/*
 * Function Name:	setMD5
 * Description:		set the message digest of file entity
 * Arguments:		thread
 * Return Value:	none
 */
static void
setMD5(thd)
	register Thread *	thd;
{
  extern void	MD5Init(MD5_CTX *);
  extern void	MD5Update(MD5_CTX *, unsigned char *, unsigned int);
  extern void	MD5Final(unsigned char [MD5_DIGEST_LENGTH], MD5_CTX *);

  MD5_CTX	context;
  size_t	len;
  int		i;
  unsigned char	md5[BUF_SIZE];
  unsigned char	digest[MD5_DIGEST_LENGTH];

  MD5Init(&context);
  while ((len = fread(md5, sizeof(char), sizeof(md5), thd->fp)) > 0) {
    MD5Update(&context, md5, len);
  }
  MD5Final(digest, &context);

  memset(md5, 0, sizeof(md5));
  for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
    (void)snprintf(md5 + i * 2, sizeof(md5), "%02x", digest[i]);
  }
  (void)fseek(thd->fp, 0L, SEEK_SET);
  thd->response.content_md5 = xstrdup(md5);
}
#endif	/* USE_CONTENT_MD5 */

#if	0
/*
 * Function Name:	setTransferEncoding
 * Description:		set the Transfer-Encoding according to TE field
 * Arguments:		thread
 * Return Value:	none
 */
static void
setTransferEncoding(thd)
	Thread *	thd;
{
  if (thd->request.te == NULL) {
    thd->response.transfer_encoding = xstrdup("chunked");
    iosPrintf(thd, "Transfer-Encoding: %s" CRLF,
		   thd->response.transfer_encoding);
  }
  else {
    /*
     * Not implemented
     */
  }
}
#endif

/*
 * Function Name:	setContentLocation
 * Description:		set Content-Location field
 * Arguments:		thread
 * Return Value:	none
 */
static void
setContentLocation(thd)
	register Thread *	thd;
{
  String	new_uri;
#if	USE_SSL
  int		n;

  if (thd->SSL.active) {
    n = xasprintf(&new_uri, "https://%s:%d%s",
		  httpd->vhost->serverName, httpd->portSSL, thd->requestURI);
  }
  else {
    n = xasprintf(&new_uri, "http://%s:%d%s",
		  httpd->vhost->serverName, httpd->port, thd->requestURI);
  }
  if (n == -1) {
    wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
  }
#else
  if (xasprintf(&new_uri, "http://%s:%d%s",
		httpd->vhost->serverName, httpd->port, thd->requestURI) == -1) {
    wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
  }
#endif	/* USE_SSL */
  thd->response.content_location = new_uri;
}
