/*****************************************************************************
 *
 * FILE:	status.c
 * DESCRIPTION:	define the status code for response
 * DATE:	Fri, Apr 27 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2001-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 2001-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: status.c,v 1.5.2.2 2002/01/27 16:09:19 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
typedef struct _responseStatus {
  int		code;	/* response status code */
  String	phrase;	/* response phrase */
} rStatus_t;


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/
/*
 *	Table of response code and phrase
 */
const rStatus_t	rsTable[] = {
  /* Informational 1xx */
  {100, "Continue"},				/* 100 */
  {101, "Switching Protocols"},			/* 101 */
  /* Successful 2xx */
  {200, "OK"},					/* 200 */
  {201, "Created"},				/* 201 */
  {202, "Accepted"},				/* 202 */
  {203, "Non-Authoritative Information"},	/* 203 */
  {204, "No Content"},				/* 204 */
  {205, "Reset Content"},			/* 205 */
  {206, "Partial Content"},			/* 206 */
  /* Redirection 3xx */
  {300, "Multiple Choices"},			/* 300 */
  {301, "Moved Permanently"},			/* 301 */
  {302, "Found"},				/* 302 */
  {303, "See Other"},				/* 303 */
  {304, "Not Modified"},			/* 304 */
  {305, "Use Proxy"},				/* 305 */
  {307, "Temporary Redirect"},			/* 307 */
  /* Client Error 4xx */
  {400, "Bad Request"},				/* 400 */
  {401, "Unauthorized"},			/* 401 */
  {402, "Payment Required"},			/* 402 */
  {403, "Forbidden"},				/* 403 */
  {404, "Not Found"},				/* 404 */
  {405, "Method Not Allowed"},			/* 405 */
  {406, "Not Acceptable"},			/* 406 */
  {407, "Proxy Authentication Required"},	/* 407 */
  {408, "Request Timeout"},			/* 408 */
  {409, "Conflict"},				/* 409 */
  {410, "Gone"},				/* 410 */
  {411, "Length Required"},			/* 411 */
  {412, "Precondition Failed"},			/* 412 */
  {413, "Request Entity Too Large"},		/* 413 */
  {414, "Request-URI Too Long"},		/* 414 */
  {415, "Unsupported Media Type"},		/* 415 */
  {416, "Requested Range Not Satisfiable"},	/* 416 */
  {417, "Expectation Failed"},			/* 417 */
  /* Server Error 5xx */
  {500, "Internal Server Error"},		/* 500 */
  {501, "Not Implemented"},			/* 501 */
  {502, "Bad Gateway"},				/* 502 */
  {503, "Service Unavailable"},			/* 503 */
  {504, "Gateway Timeout"},			/* 504 */
  {505, "HTTP Version Not Supported"}		/* 505 */
};


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	getResponsePhrase
 * Description:		get a response phrase
 * Arguments:		response status code type
 * Return Value:	phrase
 */
String
getResponsePhrase(code)
	rsCode_t	code;
{
  return rsTable[code].phrase;
}

/*
 * Function Name:	getResponseCode
 * Description:		get a response status code
 * Arguments:		response status code type
 * Return Value:	response status code
 */
int
getResponseCode(code)
	rsCode_t	code;
{
  return rsTable[code].code;
}

/*
 * Function Name:	setResponseCode
 * Description:		set a response status code
 * Arguments:		thread, code type
 * Return Value:	none
 */
void
setResponseCode(thd, code)
	register Thread *	thd;
	rsCode_t		code;
{
  thd->rsCode = code;
  switch (code) {
    /* Informational 1xx */
    case CODE_CONTINUE:				/* 100 */
    case CODE_SWITCHING_PROTOCOLS:		/* 101 */
      thd->state = SEND_READY;
      thd->ctype = C_MSG;
      break;
    /* Successfule 2xx */
    case CODE_REQUEST_OK:			/* 200 */
      thd->state = SEND_READY;
      thd->ctype = C_STD;
      break;
    case CODE_CREATED:				/* 201 */
    case CODE_ACCEPTED:				/* 202 */
    case CODE_NON_AUTH_INFO:			/* 203 */
    case CODE_NO_CONTENT:			/* 204 */
    case CODE_RESET_CONTENT:			/* 205 */
      thd->state = SEND_READY;
      thd->ctype = C_MSG;
      break;
    case CODE_PARTIAL_CONTENT:			/* 206 */
      thd->state = SEND_READY;
      thd->ctype = C_PAR;
      break;
    /* Redirection 3xx */
    case CODE_MULTIPLE_CHOICES:			/* 300 */
    case CODE_MOVED_PERMANENTLY:		/* 301 */
    case CODE_FOUND:				/* 302 */
    case CODE_SEE_OTHER:			/* 303 */
    case CODE_NOT_MODIFIED:			/* 304 */
    case CODE_USE_PROXY:			/* 305 */
    case CODE_TEMPORARY_REDIRECT:		/* 307 */
      thd->state = SEND_READY;
      thd->ctype = C_MSG;
      break;
    /* Client Error 4xx */
    case CODE_BAD_REQUEST:			/* 400 */
    case CODE_UNAUTHORIZED:			/* 401 */
    case CODE_PAYMENT_REQUIRED:			/* 402 */
    case CODE_FORBIDDEN:			/* 403 */
    case CODE_NOT_FOUND:			/* 404 */
    case CODE_METHOD_NOT_ALLOWED:		/* 405 */
    case CODE_NOT_ACCEPTABLE:			/* 406 */
    case CODE_PROXY_AUTH_REQUIRED:		/* 407 */
      thd->state = IN_ERROR;
      thd->ctype = C_MSG;
      break;
    case CODE_REQUEST_TIMEOUT:			/* 408 */
      thd->state = thd->state == SEND_READY ? IN_ERROR : TERMINATED;
      thd->ctype = C_MSG;
      break;
    case CODE_CONFLICT:				/* 409 */
    case CODE_GONE:				/* 410 */
    case CODE_LENGTH_REQUIRED:			/* 411 */
    case CODE_PRECONDITION_FAILED:		/* 412 */
    case CODE_REQ_ENTITY_TOO_LARGE:		/* 413 */
    case CODE_REQUEST_URI_TOO_LONG:		/* 414 */
    case CODE_UNSUPPORTED_MEDIA_TYPE:		/* 415 */
    case CODE_REQED_RANGE_NOT_SATISFIABLE:	/* 416 */
    case CODE_EXPECTATION_FAILED:		/* 417 */
    /* Server Error 5xx */
    case CODE_INTERNAL_SERVER_ERROR:		/* 500 */
    case CODE_NOT_IMPLEMENTED:			/* 501 */
    case CODE_BAD_GATEWAY:			/* 502 */
    case CODE_SERVICE_UNAVAILABLE:		/* 503 */
    case CODE_GATEWAY_TIMEOUT:			/* 504 */
    case CODE_HTTP_VERSION_NOT_SUPPORTED:	/* 505 */
    default:
      thd->state = IN_ERROR;
      thd->ctype = C_MSG;
      break;
  }
}

