/*****************************************************************************
 *
 * FILE:	util.c
 * DESCRIPTION:	miscellaneous/utility functions
 * DATE:	Sat, Sep 16 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 2000-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: util.c,v 1.14.2.3 2002/01/28 11:26:32 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#if	TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if	TM_IN_SYS_TIME
#include <sys/time.h>
#else
#include <time.h>
#endif	/* TM_IN_SYS_TIME */
#endif	/* TIME_WITH_SYS_TIME */
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	itoa
 * Description:		interger to string
 * Arguments:		integer
 * Return Value:	string for integer
 */
#define	INT_DIGITS	19	/* enough for 64 bit integer */
char *
itoa(i)
	long int	i;
{
  static char	buf[INT_DIGITS + 2];
  char *	p;

  memset(buf, 0, sizeof(buf));
  p = buf + INT_DIGITS + 1;	/* points to terminateing '\0' */

  if (i >= 0) {
    do {
      *--p = '0' + (i % 10);
      i	  /= 10;
    } while (i != 0);

    return p;
  }
  else {	/* i < 0 */
    do {
      *--p = '0' - (i % 10);
      i	  /= 10;
    } while (i != 0);
    *--p = '-';
  }

  return p;
}

/*
 * Function Name:	itoxa
 * Description:		interger to hex-digit string
 * Arguments:		integer
 * Return Value:	hex-digit string for integer
 */
#define	HEX_DIGITS	16	/* enough for 64 bit integer */
char *
itoxa(i)
	unsigned long int	i;
{
  static char	hex[] = "0123456789abcdef";
  static char	buf[HEX_DIGITS + 2];
  char *	p;

  memset(buf, 0, sizeof(buf));
  p = buf + HEX_DIGITS + 1;	/* points to terminateing '\0' */

  do {
    *--p = hex[i % 16];
    i	/= 16;
  } while (i != 0);

  return p;
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	skipLine
 * Description:		skip white space, '# and CR-LF
 * Arguments:		string
 * Return Value:	white space etc: TRUE, otherwise: FALSE
 */
bool_t
skipLine(s)
	String	s;
{
  switch (s[0]) {
    case '#':
    case SPC:
    case TAB:
    case CR:
    case LF:
      return TRUE;
    default:
      return FALSE;
  }
}

/*
 * Function Name:	chomp
 * Description:		remove  CRLF, CR or LF from string end
 * Arguments:		string
 * Return Value:	string
 */
String
chomp(str)
	String	str;
{
  register String	p;
  size_t		len;

  if (str == NULL) {
    return NULL;
  }

  len = strlen(str);
  p   = &str[len - 1];
  if (*p == LF) {
    *p-- = EOL;
  }
  if (*p == CR) {
    *p-- = EOL;
  }

  return str;
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	decodeEscapedURI
 * Description:		decode an escaped character
 * Arguments:		string
 * Return Value:	decoded string
 * Reference:		RFC 2396
 *			Uniform Resource Identifiers (URI): Generic Syntax
 */
String
decodeEscapedURI(s)
	String	s;
{
  char			tmp[MAX_URI_LEN];
  register String	os  = tmp;
  register String	ret = s;
  char			c, c1;

  /* copy original requst URI into working buffer */
  memset(tmp, 0, sizeof(tmp));
  memcpy(tmp, s, sizeof(tmp));

  while ((c = *os) != EOL) {
    /* escaped URI */
    if (c == '%') {
      os++;
      if ((c = *os++) && (c1 = *os++)) {
	*s =	((c  >= 'A' ? ((c  & 0xdf) - 'A') + 10 : c  - '0') * 16) +
		((c1 >= 'A' ? ((c1 & 0xdf) - 'A') + 10 : c1 - '0'));
	c = *s;
	os--;	/* back one word */
      }
      else {
        return ret;
      }
    }
    /* escape meta characters for cross-site scripting */
    if (c == '<') {
      *s++ = '&'; *s++ = 'l'; *s++ = 't'; *s++ = ';';
    }
    else if (c == '>') {
      *s++ = '&'; *s++ = 'g'; *s++ = 't'; *s++ = ';';
    }
    else if (c == '"') {
      *s++ = '&'; *s++ = 'q'; *s++ = 'u'; *s++ = 'o'; *s++ = 't'; *s++ = ';';
    }
    else if (c == '&') {
      *s++ = '&'; *s++ = 'a'; *s++ = 'm'; *s++ = 'p'; *s++ = ';';
    }
    /* normal characters */
    else {
      *s++ = c;
    }
    os++;
  }
  *s = EOL;

  return ret;
}

/*****************************************************************************
 *****************************************************************************/

/*****************************************************************************
 *
 * Replace string utilities in string.h
 *
 *****************************************************************************/

/*
 * Function Name:	xstrncpy
 * Description:		boundary checked strncpy()
 * Arguments:		dst, src, size
 * Return Value:	success: same as strncpy(), otherwise: NULL
 */
String
xstrncpy(dst, src, size)
	String		dst;
	const String	src;
	size_t		size;	/* size of dst buffer */
{
  String	dp;	/* working pointer */
  String	end;	/* end pointer */

  if (size == 0) {	/* invalid dst buffer size */
    return NULL;
  }

  if (src == NULL) {	/* invalid src pointer */
    return NULL;
  }

  dp  = dst;
  end = dst + size - 1;

  for (; dp < end; dp++, src++) {
    if ((*dp = *src) == NULL) {
      return dp;
    }
  }
  *dp = EOL;

  return dp;
}


/*****************************************************************************
 *****************************************************************************/

#ifndef	HAVE_USLEEP
/*
 * Function Name:	usleep
 * Description:		suspend process execution for an interval measurement
 *			in microseconds
 * Arguments:		microseconds
 * Return Value:	Successful: 0, otherwise: -1
 */
int
usleep(useconds)
	unsigned int	useconds;
{
  struct timeval	sleep_time;

  sleep_time.tv_usec = (useconds % USECONDS_BASE);
  sleep_time.tv_sec  = useconds / USECONDS_BASE;

  return select(0, NULL, NULL, NULL, &sleep_time);
}
#endif	/* HAVE_USLEEP */


/*****************************************************************************
 *****************************************************************************/

#if	0
/*
 * Function Name:	hash
 * Description:		calculate hash value
 * Arguments:		string, string length
 * Return Value:	hash value
 * Note:		The original code is in the sdbm sources.
 */
unsigned int
hash(str, len)
	register String	str;
	register int	len;
{
  register unsigned long	n = 0;
  while (len-- > 0) {
    n = (*str++ + 65599 * n);
  }
  return n;
}
#endif

