/*****************************************************************************
 *
 * FILE:	vhost.c
 * DESCRIPTION:	virtual host
 * DATE:	Sun, Apr 22 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2001-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 2001-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: vhost.c,v 1.13.2.3 2002/01/30 13:26:33 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	newVirtualHost
 * Description:		make a new virtual host in vhostList
 * Arguments:		current virtual host pointer
 * Return Value:	new virutal host pointer
 */
VirtualHost *
newVirtualHost(curt)
	VirtualHost *	curt;
{
  VirtualHost *	new;
  int		i;

  new = (VirtualHost *)xcalloc(1, sizeof(VirtualHost));
  if (new == NULL) {
    die(ERR_OUT_OF_MEMORY);
  }

  new->serverAdmin	= NULL;		/* ServerAdmin	*/
  new->serverName	= NULL;		/* ServerName	*/
  new->errorLog.path	= NULL;		/* ErrorLog	*/
  new->errorLog.fp	= NULL;		/* ErrorLog	*/
  new->accessLog.path	= NULL;		/* AccessLog	*/
  new->accessLog.fp	= NULL;		/* AccessLog	*/
  new->docRoot		= NULL;		/* DocumentRoot	*/
  new->userDir		= NULL;		/* UserDir	*/
  new->dirIndex		= NULL;		/* DirectoryIndex */
  new->symlinks		= FALSE;	/* FollowSymLinks */
  new->defaultType	= NULL;		/* DefaultType	*/

  for (i = 0; i < ALIAS_TABLE_SIZE; i++) {
    new->aliasTable[i] = NULL;
  }

  if (curt == NULL) {
    new->next = curt;
  }
  else {
    new->next	= curt->next;
    curt->next	= new;
  }

  return new;
}

/*
 * Function Name:	deleteVirtualHosts
 * Description:		delete virtual hosts in vhostList
 * Arguments:		none
 * Return Value:	none
 */
void
deleteVirtualHosts(void)
{
  register VirtualHost *	vp;
  register VirtualHost *	tmp;

  for (vp = httpd->host; tmp != NULL; vp = tmp) {
    int		i;

    tmp = vp->next;
    for (i = 0; i < ALIAS_TABLE_SIZE; i++) {
      register Alias *	ap;
      register Alias *	tmp0;

      if (vp->aliasTable[i] == NULL) {
	continue;
      }
      for (ap = vp->aliasTable[i]; ap != NULL; ap = tmp0) {
	tmp0 = ap->next;
	XFree((Alias *)ap->realName);
	XFree((Alias *)ap->fakeName);
	XFree((Alias *)ap);
      }
    }
    XFree(vp->errorLog.path);
    XFree(vp->accessLog.path);
    XFree(vp->serverAdmin);
    XFree(vp->serverName);
    XFree(vp->docRoot);
    XFree(vp->userDir);
    XFree(vp->dirIndex);
    XFree(vp->defaultType);
    XFree(vp);
  }
}

/*
 * Function Name:	verifyVirtualHosts
 * Description:		verify parameters of virtual hosts
 * Arguments:		none
 * Return Value:	none
 */
void
verifyVirtualHosts(void)
{
  register VirtualHost *	vhost;

  for (vhost = httpd->host->next; vhost != NULL; vhost = vhost->next) {
    if (vhost->serverAdmin == NULL) {
      die("No ServerAdmin for VirtualHost in wyvern.conf\n");
    }
    if (vhost->docRoot == NULL) {
      die("No DocRoot for VirtualHost in wyvern.conf\n");
    }

    if (vhost->userDir == NULL) {
      vhost->userDir = httpd->host->userDir;
    }
    if (vhost->dirIndex == NULL) {
      vhost->dirIndex = httpd->host->dirIndex;
    }
    if (vhost->defaultType == NULL) {
      vhost->defaultType = httpd->host->defaultType;
    }
  }
}

/*
 * Function Name:	switchVirtualHost
 * Description:		search virtual host and set current virtual host
 * Arguments:		thread
 * Return Value:	Successful: TRUE, otherwise: FALSE
 */
bool_t
switchVirtualHost(thd)
	register Thread *	thd;
{
  char				host[MAX_HOSTNAME_LEN];
  size_t			len;
  register VirtualHost *	vp;
  String			ipaddr = inet_ntoa(httpd->socket.addr.sin_addr);

  memset(host, 0, sizeof(host));
  /* get hostname from Request-URI */
  if (thd->requestURI[0] != '/') {	/* absoluteURI */
    String		tmp = xstrdup(thd->requestURI);
    register String	sp  = strstr(tmp, "://");
    register String	ep;
    if (sp != NULL) {
      sp += 3;
      ep  = strchr(sp, '/');
      if (ep != NULL) {
	*ep = EOL;
      }
      ep = strchr(sp, ':');
      if (ep != NULL) {
	*ep = EOL;
      }
    }
    xstrncpy(host, sp, sizeof(host));
    xfree((String)tmp);
  }
  /* get hostname from Host field in request header */
  else if (thd->request.host != NULL) {
    String		tmp = xstrdup(thd->request.host);
    register String	sp  = strchr(tmp, ':');
    if (sp != NULL) {
      *sp = EOL;
    }
    xstrncpy(host, tmp, sizeof(host));
    xfree((String)tmp);
  }
  else {	/* no virtual hosting */
    /* switch default log files */
#ifndef	USE_SYSLOG
    httpd->errorLog	= httpd->host->errorLog.fp;
#endif	/* USE_SYSLOG */
    httpd->accessLog	= httpd->host->accessLog.fp;
    httpd->vhost	= httpd->host;
    return TRUE;
  }

  /* find the virtual host */
  len = strlen(host);
  for (vp = httpd->host; vp != NULL; vp = vp->next) {
    if (strncasecmp(vp->serverName, host, len) == 0) {
#ifndef	USE_SYSLOG
      httpd->errorLog	= vp->errorLog.fp;
#endif	/* USE_SYSLOG */
      httpd->accessLog	= vp->accessLog.fp;
      httpd->vhost	= vp;
      return TRUE;
    }
  }

  /* bind address 0.0.0.0 and localhost/127.0.0.1 are specail. */
  if (strcmp(ipaddr, host) == 0 ||
      strcmp(ipaddr, "0.0.0.0") == 0 ||
      strcmp(host, "localhost") == 0 || strcmp(host, "127.0.0.1") == 0) {
#ifndef	USE_SYSLOG
    httpd->errorLog	= httpd->host->errorLog.fp;
#endif	/* USE_SYSLOG */
    httpd->accessLog	= httpd->host->accessLog.fp;
    httpd->vhost	= httpd->host;
    return TRUE;
  }

  /* Why Bad Request? See RFC2616 Section 5.2! */
  setResponseCode(thd, CODE_BAD_REQUEST);
  return FALSE;
}
