/*****************************************************************************
 *
 * FILE:	wyvern.h
 * DESCRIPTION:	global header file for wyvern
 * DATE:	Thu, Aug 26 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wyvern.h,v 1.69.2.5 2002/01/29 10:24:25 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_wyvern_h
#define	_wyvern_h

#ifdef	MAIN_MODULE
#define	EXTERN
#else
#define	EXTERN extern
#endif	/* MAIN_MODULE */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#if	USE_SSL
#include <openssl/ssl.h>
#endif	/* USE_SSL */
#if	defined(USE_SYSLOG) && defined(HAVE_SYSLOG_H)
#include <syslog.h>
#endif	/* USE_SYSLOG */
#include "common.h"
#include "version.h"


/*****************************************************************************
 *****************************************************************************/

#define	DEF_CONFIG	"conf/wyvern.conf"	/* configuration file */
#define	DEF_PORT	80			/* Port	*/
#define	DEF_SECURE_PORT	443			/* SecurePort via SSL */
#define	DEF_MAX_REQUEST	8			/* MaxRequest */
#define	DEF_MIN_REQUEST	2			/* Minimum Request */
#define	DEF_TIMEOUT	30			/* Timeout (sec.) */
#define	DEF_KA_TIMEOUT	6			/* KeepAliveTimeout (sec.) */
#define	DEF_KA_MAX	10			/* KeepAliveMax	*/
#define	DEF_DIRINDEX	"index.html"		/* DirectoryIndex */
#define	DEF_DEFTYPE	"text/plain"		/* DefaultType */

#define	DEF_LOG_DIR	"logs"			/* directory to store logs */
#define	DEF_TMP_DIR	".tmp"			/* working directory*/
#define	DEF_CACHE_DIR	"cache"			/* Directory Listing*/
#define	DEF_ETC_DIR	"etc"			/* password etc. */
#if	0
#define	DEF_DELETE_DIR	"deleted"	/* for DELETE method */
#endif
#define	DEF_PID_FILE	DEF_LOG_DIR ## "/wyvern.pid"	/* PidFile */

#include "wpath.h"		/* include paths for access control files */

#define	HTTP_VERSION	"HTTP/1.0"
#define	CGI_VERSION	"CGI/1.1"
#define	WYVERN_METHODS	"HEAD, GET, POST"
#define	RANGE_UNIT	"bytes"
#define	TYPE_TEXT_HTML	"text/html"

#define	UID_ROOT	0	/* super user id in /etc/passwd */
#define	UID_WHEEL	0	/* super user's group id in /etc/group */
#define	UID_NOBODY	65534	/* nobody user id in /etc/passwd */
#define	UID_NOGROUP	65533	/* nogroup id in /etc/group */

#if	0
/* embedded simple search engine */
#define	SEARCH_CGI	"/ecgi-bin/search.cgi"
#endif

#define	MAX_HEADER_SIZE		4096	/*  request header field */
#define	RESPONSE_BUFSIZE	2048	/* response header field */
#define	SOCKET_BUFSIZE		4096

#define	MAX_TIMEOUT		(60*60*24)	/* 24 hour */
#define	MAX_CGI_ENVNO		(34+1)	/* the max number of CGI environment */

#define	ALIAS_TABLE_SIZE	11	/* Redirect/Alias/ScriptAlias	*/
#define	ACCESS_TABLE_SIZE	7	/* hosts.allow/hosts.deny file	*/
#define	MIME_TABLE_SIZE		79	/* mime.types file	*/
#define	USER_TABLE_SIZE		13	/* user's home directory cache	*/
#define	DIR_TABLE_SIZE		17	/* directory indexing cache	*/
#define	ERRDOC_TABLE_SIZE	5	/* customizable error documents */

#define	CGI_MIME_TYPE		"application/x-httpd-cgi"
#define	SSI_MIME_TYPE		"text/x-server-parsed-html"
#define	DEFAULT_PATH		"/bin:/usr/bin:/usr/local/bin"

#define	MAX_URI_LEN		256	/* maximum URI length */
#define	MAX_HOSTNAME_LEN	128
#ifdef	USE_IPV6
#define	MAX_HOSTADDR_LEN	INET6_ADDRSTRLEN	/* is longer than INET_ADDRSTRLEN */
#else
#define	MAX_HOSTADDR_LEN	32
#endif	/* USE_IPV6 */
#define	MAX_FILENAME_LEN	80
#define	MAX_PATH_LEN		512
#define	MAX_NAME_LEN		10	/* maximum user name length */
#define	MAX_DIR_ENTRY		1024

#define	MAX_FILE_SIZE		(3*1024*1024)	/* file resource limit: 3MB */

#define	CGI_ERR_STATUS		(99)

#define	IO_USLEEP_TIME		(1000)	/* 0.01 seconds */
#define	SSL_USLEEP_TIME		(100000)/* 0.1 seconds */

#define	TILDE_OFFSET		2	/* offset of '~' */


/* macro to get the current time of RFC822 */
#include <time.h>
#if	TIME_WITH_SYS_TIME
#include <sys/time.h>
#endif	/* TIME_WITH_SYS_TIME */
#define	getCurrentTime		getTimeOfRFC822(time(NULL))

#define	XFree(x)		{xfree((x));(x)=NULL;}

/* If you use syslog(), wyvern replaces wvSyslog with syslog. */
#if	USE_SYSLOG
#define	wvSyslog	syslog
#endif	/* USE_SYSLOG */

/* error messages */
#define	ERR_OUT_OF_MEMORY	"failed to allocate memory\n"


/*****************************************************************************
 *****************************************************************************/
#include "status.h"
#include "mesg.h"


/*****************************************************************************
 *****************************************************************************/

typedef enum {	/* method type		*/
  M_UNKNOWN = 0,
  M_HEAD,	/* See RFC2616 Section 9.4 (HEAD request)	*/
  M_GET,	/* See RFC2616 Section 9.3 (GET  request)	*/
  M_POST,	/* See RFC2616 Section 9.5 (POST request)	*/
  M_OPTIONS,	/* See RFC2616 Section 9.2 (OPTIONS request)	*/
  M_PUT,	/* See RFC2616 Section 9.6 (PUT request)	*/
  M_DELETE,	/* See RFC2616 Section 9.7 (DELETE request)	*/
  M_MOVE,	/* extension method of Netscape Communicator	*/
  M_TRACE,	/* See RFC2616 Section 9.8 (TRACE request)	*/
  M_CONNECT,	/* See RFC2616 Section 9.9 (CONNECT request	*/
  M_SEARCH	/* Wyvern original method (SEARCH request)	*/
} method_t;


typedef struct _Request {	/* HTTP request structure	*/
  /* general header	*/
  String	connection;		/* See RFC2616 Section 14.10	*/
  String	via;			/* See RFC2616 Section 14.45	*/
#if	SUPPORT_ALL_REQUEST_HEADER
  String	cache_control;		/* See RFC2616 Section 14.9	*/
  String	pragma;			/* See RFC2616 Section 14.32	*/
  String	trailer;		/* See RFC2616 Section 14.40	*/
  String	transfer_encoding;	/* See RFC2616 Section 14.41	*/
#endif	/* SUPPORT_ALL_REQUEST_HEADER */

  /* request header	*/
  String	accept;			/* See RFC2616 Section 14.1	*/
  String	accept_charset;		/* See RFC2616 Section 14.2	*/
  String	accept_encoding;	/* See RFC2616 Section 14.3	*/
  String	accept_language;	/* See RFC2616 Section 14.4	*/
  String	authorization;		/* See RFC2616 Section 14.8	*/
  String	from;			/* See RFC2616 Section 14.22	*/
  String	host;			/* See RFC2616 Section 14.23	*/
  String	if_match;		/* See RFC2616 Section 14.24	*/
  String	if_modified_since;	/* See RFC2616 Section 14.25	*/
  String	if_none_match;		/* See RFC2616 Section 14.26	*/
  String	if_range;		/* See RFC2616 Section 14.27	*/
  String	if_unmodified_since;	/* See RFC2616 Section 14.28	*/
  String	range;			/* See RFC2616 Section 14.35	*/
  String	referer;		/* See RFC2616 Section 14.36	*/
  String	user_agent;		/* See RFC2616 Section 14.43	*/
#if	SUPPORT_ALL_REQUEST_HEADER
  String	expect;			/* See RFC2616 Section 14.20	*/
  String	max_forwards;		/* See RFC2616 Section 14.31	*/
  String	proxy_authorization;	/* See RFC2616 Section 14.34	*/
  String	te;			/* See RFC2616 Section 14.39	*/
#endif	/* SUPPORT_ALL_REQUEST_HEADER */

  /* entity header	*/
  String	content_encoding;	/* See RFC2616 Section 14.11	*/
  String	content_language;	/* See RFC2616 Section 14.12	*/
  unsigned int	content_length;		/* See RFC2616 Section 14.13	*/
  String	content_md5;		/* See RFC2616 Section 14.15	*/
  String	content_type;		/* See RFC2616 Section 14.17	*/

  /* optional header */
  String	cookie;			/* See RFC2109	*/
	/* cf. http://www.netscape.com/newsref/std/cookie_spec.html	*/

  /* extension header */
  String	x_forwarded_for;/* Squid uses this request header */
} Request;

typedef struct _Response {	/* HTTP response structure	*/
  /* general header	*/
  String	connection;		/* See RFC2616 Section 14.10	*/
  String	date;			/* See RFC2616 Section 14.18	*/
#if	SUPPORT_ALL_RESPONSE_HEADER
  String	trailer;		/* See RFC2616 Section 14.40	*/
  String	transfer_encoding;	/* See RFC2616 Section 14.41	*/
  String	upgrade;		/* See RFC2616 Section 14.42	*/
  String	via;			/* See RFC2616 Section 14.45	*/
  String	warning;		/* See RFC2616 Section 14.46	*/
#endif	/* SUPPORT_ALL_RESPONSE_HEADER */

  /* response header	*/
  String	accept_ranges;		/* See RFC2616 Section 14.5	*/
  String	etag;			/* See RFC2616 Section 14.19	*/
  String	location;		/* See RFC2616 Section 14.30	*/
  String	server;			/* See RFC2616 Section 14.38	*/
  String	www_authenticate;	/* See RFC2616 Section 14.47	*/
#if	SUPPORT_ALL_RESPONSE_HEADER
  String	age;			/* See RFC2616 Section 14.6	*/
  String	proxy_authenticate;	/* See RFC2616 Section 14.33	*/
  String	retry_after;		/* See RFC2616 Section 14.37	*/
  String	vary;			/* See RFC2616 Section 14.44	*/
#endif	/* SUPPORT_ALL_RESPONSE_HEADER */

  /* entity header	*/
  String	allow;			/* See RFC2616 Section 14.7	*/
  String	last_modified;		/* See RFC2616 Section 14.29	*/
  String	content_encoding;	/* See RFC2616 Section 14.11	*/
  unsigned int	content_length;		/* See RFC2616 Section 14.13	*/
  String	content_location;	/* See RFC2616 Section 14.14	*/
  String	content_md5;		/* See RFC2616 Section 14.15	*/
  String	content_range;		/* See RFC2616 Section 14.16	*/
  String	content_type;		/* See RFC2616 Section 14.17	*/
#if	SUPPORT_ALL_RESPONSE_HEADER
  String	content_language;	/* See RFC2616 Section 14.12	*/
  String	expires;		/* See RFC2616 Section 14.21	*/
#endif	/* SUPPORT_ALL_RESPONSE_HEADER */

  /* optional header */
  String	set_cookie;		/* refer the same as 'request cookie' */
} Response;

/*****************************************************************************/

#if	USE_SSL
typedef enum {
  SSL_ERROR = -1,
  SSL_OK,
  SSL_RETRY
} ssl_t;
#endif	/* USE_SSL */

typedef enum {	/* HTTP version type	*/
  HTTP_09,	/* version 0.9	*/
  HTTP_10,	/* version 1.0	*/
  HTTP_11	/* version 1.1	*/
} http_t;

typedef enum {	/* KeepAlive state type	*/
  KA_DISABLED	= 0,
  KA_OK		= 1,
  KA_CLOSE,
  KA_CONTINUE
} keepalive_t;

typedef enum {	/* thread state type	*/
  UNUSED,
  IN_ERROR,
  RECV_READY,
  SEND_READY,
  CGI_READY,
  CGI_WAITING,
  SSI_READY,
  EMB_READY,
#if	USE_SSL
  SSL_READY,
#endif	/* USE_SSL */
  NOW_SENDING,
  TERMINATED,
  DETACHED
} threadState_t;

typedef enum {	/* response content type */
  C_UNK,	/* unknown content type */
  C_STD,	/* standard html content */
  C_PAR,	/* partial html content */
  C_CGI,	/* CGI generated content */
  C_SSI,	/* SSI generated content */
  C_DIR,	/* Directory listing content */
  C_MSG,	/* Error/Status message content */
  C_EMB		/* embedded content */
} content_t;

typedef struct _Range {	/* range unit structure */
  int	first;		/* first range for Range: */
  int	last;		/* last  range for Range: */
} Range;

typedef struct _Parts {	/* partial content structure */
  int	fsize;		/* response content length */
  int	bsize;		/* read buffer size */
  int	sum;		/* total sent bytes */
} Parts;

typedef struct _StrList {	/* string list */
  String		text;	/* text */
  struct _StrList *	next;	/* next list */
} StrList;

typedef struct _CGI {
  int		fd;		/* file descriptor	*/
  FILE *	fp;		/* file pointer	*/
  String	post_tmp;	/* temporary file for POST method	*/
  pid_t		pid;		/* pid of CGI program	*/
  String	path_info;	/* for PATH_INFO	*/
  String	query_string;	/* for QUERY_STRING	*/
  int		envno;		/* numbers of CGI environments	*/
  String *	env;		/* environments for CGI program	*/
  String	status;		/* CGI has own response status line */
  StrList *	headers;	/* response header by CGI program */
} CGI;

/*
 * wyvern thread structure
 */
typedef struct _Thread {
#if	DEBUG
  int		no;	/* thread number */
#endif	/* DEBUG */
  /* client's socket descriptor */
  int		sd;

  struct _stream {
    bool_t	update;	/* flag for buffer status */
    int		total;	/* sum of character in buf */
    int		next;	/* next character inbuf */
    char	buf[SOCKET_BUFSIZE];	/* I/O buffer */
  } ios;

  /* client's hotname and IP address */
  struct _Client {
    int		port;	/* remote port */
    String	name;	/* remote hostname */
    String	ip;	/* remote IP address */
#if	USE_IPV6
    struct sockaddr_in6	addr;
#else
    struct sockaddr_in	addr;
#endif	/* USE_IPV6 */
  } client;

  /* Keep-Alive: 0-disabled 2-close 3-continue */
  struct _KeepAlive {
    keepalive_t	state;	/* state */
    int		max;	/* maximum requests per connection */
    int		timeout;/* seconds to wait for another request */
  } keepAlive;

  /* last request/response time */
  time_t	atime;

  /* thread state code	*/
  threadState_t	state;

  /* response status */
  rsCode_t	rsCode;	

  /* HTTP method */
  struct _Method {
    method_t	type;		/* method type */
    char	name[10];	/* method name */
  } method;

  /* HTTP protocol version */
  struct _HTTP {
    http_t	version;	/* version */
    char	protocol[8+1];	/* protocol */
  } http;

  /* request URI */
  char		requestURI[MAX_URI_LEN];

  FILE *	fp;		/* fp for data file	*/
  String	path;		/* actual path of data file	*/
  String	userDir;	/* path of user directory (i.e. public_html) */
  String	userHome;	/* user home directory */
  String	userName;	/* owner of UserDir	*/
  String	identUser;	/* identification user	*/
  String	authUser;	/* authenticated user	*/
  struct stat	file;		/* file status structure	*/
  String	mtype;		/* mime type	*/
  content_t	ctype;		/* response content type */
  Range		range;		/* range unit for partial request */
  Parts		parts;		/* content state for partial request */

  Request	request;	/*  request header	*/
  Response	response;	/* response header	*/

  CGI		cgi;		/* CGI */

#if	HAVE_ZLIB_H
  double	gzippct;	/* compression ratio using gzip.mod */
#endif	/* HAVE_ZLIB_H */

#if	USE_SSL
  struct _ssl {
    bool_t		active;		/* connection status */
    SSL *		ssl;		/* SSL connection */
  } SSL;
#endif	/* USE_SSL */

  struct _Thread *	next;	/* next thread	*/
  struct _Thread *	prev;	/* previous thread */
} Thread;

/*****************************************************************************/

typedef struct _Log {	/* logfile structure	*/
  String	path;	/* path for log file	*/
  FILE *	fp;	/* file pointer for log	*/
} Log;

typedef enum {	/* alias type	*/
  A_REDIRECT,	/* Redirect	*/
  A_ALIAS,	/* Alias	*/
  A_SCRIPT	/* ScriptAlias	*/
} alias_t;

typedef struct _Alias {			/* alias structure	*/
  alias_t		type;		/* alias type		*/
  String		fakeName;	/* URI path to file	*/
  String		realName;	/* actual path to file	*/
  size_t		fakeLen;	/* strlen of fakeName	*/
  size_t		realLen;	/* strlen of realName	*/
  struct _Alias	*	next;
} Alias;

typedef struct _UserInfo {
  String		name;		/* owner of UserDir	*/
  String		home;		/* home directory	*/
  time_t		lastModified;	/* last modified time	*/
  struct _UserInfo *	prev;
  struct _UserInfo *	next;
} UserInfo;

#include "module.h"	/* include module management structure of wyvern */

typedef struct _VirtualHost {	/* virtual host structure */
  String	serverName;	/* ServerName	*/
  String	serverAdmin;	/* ServerAdmin	*/
  String	docRoot;	/* DocumentRoot	*/
  Log		errorLog;	/* ErrorLog	*/
  Log		accessLog;	/* AccessLog	*/
  /**/
  String	userDir;	/* UserDir		*/
  String	dirIndex;	/* DirectoryIndex	*/
  bool_t	symlinks;	/* FollowSymLinks	*/
  String	defaultType;	/* DefaultType		*/
  /**/
  Alias *	aliasTable[ALIAS_TABLE_SIZE];	/* alias management table */
  /**/
  struct _VirtualHost *	next;
} VirtualHost;

/*
 * wyvern HTTP server structure (for wyvern.conf)
 */
typedef struct _Server {
  /* socket manager */
  struct _Socket {
    int			soc;	/* socket for accept	*/
#if	USE_SSL
    int			ssl;	/* socket for SSL	*/
#endif	/* USE_SSL */
#ifdef	USE_IPV6
    struct sockaddr_in6	addr;	/* socket address of wyvern */
#else
    struct sockaddr_in	addr;	/* socket address of wyvern */
#endif	/* USE_IPV6 */
  } socket;

  /* following variables defined by wyvern.conf */
  unsigned int	port;		/* Port		*/
#if	USE_SSL
  unsigned int	portSSL;	/* PortSSL	*/
#endif	/* USE_SSL */
  /**/
  int		maxRequest;	/* MaxRequest	*/
  uid_t		uid;		/* User		*/
  gid_t		gid;		/* Group	*/
  pid_t		pid;		/* Wyvern PID	*/
  String	pidFile;	/* PidFile	*/
  FILE *	errorLog;	/* ErrorLog	*/
  FILE *	accessLog;	/* AccessLog	*/
  String	serverRoot;	/* ServerRoot	*/
  time_t	timeout;	/* Timeout	*/
  time_t	ka_timeout;	/* KeepAliveTimeout	*/
  unsigned int	ka_max;		/* KeepAliveMax		*/
  String	bindAddress;	/* BindAddress	*/
  /**/
  Modules	mod;		/* module manager */
  /**/
  VirtualHost *	vhostList;	/* virtual host manager with list */
  VirtualHost *	vhost;		/* current virtual host */
  VirtualHost *	host;		/* original server host */
  /**/
  String	configFile;	/* wyvern.conf path	*/
  String	cacheDir;	/* CacheDir	*/
  String	tmpDir;		/* temporary working directory	*/
  /**/
  int		envno;		/* numbers of script environments */
  String *	env;		/* script environments */
  /**/
  bool_t	reboot;		/* flag whether server will reboot */
} Server;

/*****************************************************************************/

typedef struct _Hash {	/* hash structure	*/
  String		key;
  String		value;
  struct _Hash *	next;
} Hash;

/*****************************************************************************/

#ifndef	USE_SYSLOG
typedef enum {	/* log type */
  LOG_FATAL,
  LOG_EMERG,	/* system is unusable */
  LOG_ALERT,	/* action must be taken immediately */
  LOG_CRIT,	/* critical conditions */
  LOG_ERR,	/* error conditions */
  LOG_WARNING,	/* warning conditions */
  LOG_NOTICE,	/* normal but significant condition */
  LOG_INFO,	/* informational */
  LOG_DEBUG,	/* debug-level messages */
  LOG_HTTP	/* http log message */
} log_t;
#endif	/* USE_SYSLOG */


/*****************************************************************************
 *****************************************************************************/

EXTERN Server	*	httpd;	/* server body	*/

#endif	/* _wyvern_h */
