/*****************************************************************************
 *
 * FILE:	wpasswd.c
 * DESCRIPTION:	a front end program for '.wpasswd'
 * DATE:	Sat, Dec 12 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wpasswd.c,v 1.8 2002/01/15 15:09:49 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <termios.h>
#include <pwd.h>
#include "common.h"
#include "wpath.h"
#include "md5.h"
#include "wdbm.h"


/*****************************************************************************
 *
 *	declaration of macros and structures
 *
 *****************************************************************************/
const char	Program[]   = "wpasswd";
const char	Copyright[] = "Copyright (C) 1999-2002 Kouichi ABE (WALL)";

#define	DEF_USER_DIR	"public_html"


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static String	encryption(String);
static String	chomp(String);
static void	registration(String, String, String);
static void	reception(String);


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	encryption
 * DESCRIPTION:		make a new password
 * ARGUMENTS:		password (plain text)
 * RETURN VALUE:	a crypted password
 */
static String
encryption(plainPassword)
	String	plainPassword;
{
  MD5_CTX	context;
  size_t	len = strlen(plainPassword);
  int		i;
  unsigned char	md5[TINY_BUFSIZE+1];
  unsigned char	digest[16];

  MD5Init(&context);
  MD5Update(&context, plainPassword, len);
  MD5Final(digest, &context);

  memset(md5, 0, sizeof(md5));
  for (i = 0; i < 16; i++) {
    (void)snprintf(md5 + i * 2, sizeof(md5), "%02x", digest[i]);
  }

  return strdup(md5);
}

/*
 * Function Name:	chomp
 * Description:		remove  CRLF, CR or LF from string end
 * Arguments:		string
 * Return Value:	string
 */
static String
chomp(str)
	String	str;
{
  String	p;
  size_t	len;

  if (str == NULL) {
    return NULL;
  }

  len = strlen(str);
  p   = &str[len - 1];
  if (*p == LF) {
    *p-- = EOL;
  }
  if (*p == CR) {
    *p-- = EOL;
  }

  return str;
}

/*
 * FUNCTION NAME:	registration
 * DESCRIPTION:		register user and password
 * ARGUMENTS:		username, password, path of '.wpasswd'
 * RETURN VALUE:	none
 */
static void
registration(name, pass, file)
	String	name;
	String	pass;
	String	file;
{
  String	passwd;
  WDBM_FILE	dp;
  wdatum	key;
  wdatum	val;
  int		ret;

  passwd = encryption(chomp(pass));
  if (passwd == NULL) {
    perror("encryption");
    exit(1);
  }

  dp = wdbm_open(file, WDBM_WRCREAT, 0600);
  if (dp == NULL) {
    fprintf(stderr, "%s: failed to open %s\n", Program, file);
    exit(1);
  }
  key.dptr	= chomp(name);
  key.dsize	= strlen(key.dptr);
  val.dptr	= passwd;
  val.dsize	= strlen(val.dptr);

  ret = wdbm_store(dp, key, val, WDBM_REPLACE);
  if (ret == -1) {
    perror("wdbm_store");
    exit(1);
  }
  wdbm_close(dp);
#if	DEBUG
  fprintf(stderr, "[%s]\t%s:%s\n", file, name, passwd);
#endif	/* DEBUG */

  free((String)passwd);
}

/*
 * FUNCTION NAME:	reception
 * DESCRIPTION:		recept user and password
 * ARGUMENTS:		path of wyvern password file
 * RETURN VALUE:	none
 */
static void
reception(path)
	String	path;
{
#define	USERNAME_SIZE	(16+1)
#define	PASSWORD_SIZE	(48+1)
  char			user[USERNAME_SIZE];
  char			pass[PASSWORD_SIZE];
  char			conf[PASSWORD_SIZE];
  struct termios	tc_old, tc_new;


  fputs("Type EOF to quit\n", stderr);
  tcgetattr(fileno(stdin), &tc_old);
  for (;;) {
    memset(user, 0, sizeof(user));
    fputs("Username: ", stdout);
    if (fgets(user, sizeof(user), stdin) == NULL) {
      exit(0);
    }

    memset(pass, 0, sizeof(pass));
    fputs("Password: ", stdout);
    tc_new = tc_old;
    tc_new.c_lflag &= ~ECHO;
    tcsetattr(fileno(stdin), TCSAFLUSH, &tc_new);
    if (fgets(pass, sizeof(pass), stdin) == NULL) {
      exit(0);
    }
    putchar(LF);

    memset(conf, 0, sizeof(conf));
    fputs("Retype password: ", stdout);
    if (fgets(conf, sizeof(conf), stdin) == NULL) {
      exit(0);
    }
    putchar(LF);
    tcsetattr(fileno(stdin), TCSANOW, &tc_old);

    if (strlen(pass) == 1) {
      fputs("Empty Password; try again, EOF to quit\n", stderr);
    }
    else if (strcmp(pass, conf) == 0) {
      registration(user, pass, path);
    }
    else {
      fputs("Mismatch; try again, EOF to quit\n", stderr);
    }
  }
}


/*
 *	MAIN FUNCTION
 */
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  int			ch;
  String		userDir = DEF_USER_DIR;
  char			path[BUFSIZ];	/* path of '.wpasswd' */
  struct passwd *	pwd;

  memset(path, 0, sizeof(path));
  while ((ch = getopt(argc, argv, "sd:")) != -1) {
    switch (ch) {
      case 's':
	snprintf(path, sizeof(path), "%s/%s", SERVER_ROOT, GLOBAL_HOSTS_PASSWD);
	break;
      case 'd':
	userDir = optarg;
	break;
      case '?':
      default:
	fprintf(stderr, "%s\n"
			"usage: %s [-s | -d user_dir]\n",
			Copyright,
			Program);
	exit(1);
    }
  }
  argc -= optind;
  argv += optind;

  if (path[0] == EOL) {
    /* get user home directory */
    pwd = getpwuid(getuid());
    if (pwd == NULL) {
      perror("getpwuid");
      exit(1);
    }
    snprintf(path, sizeof(path), "%s/%s/%s", pwd->pw_dir, userDir, USER_PASSWD);
  }

  reception(path);

  return 0;
}

